/*****************************************************************************
* Copyright (c) Future Technology Devices International 2014
* propriety of Future Technology devices International.
*
* Software License Agreement
*
* This code is provided as an example only and is not guaranteed by FTDI. 
* FTDI accept no responsibility for any issues resulting from its use. 
* The developer of the final application incorporating any parts of this 
* sample project is responsible for ensuring its safe and correct operation 
* and for any consequences resulting from its use.
*****************************************************************************/
/**
* @file                           FT_App_Mainmenu
* @brief                          MainMenu application with background animation
* @version                        0.1.0
* @date                           2014/05/30
*
*/


/* Sample application to demonstrate FT800 primitives, widgets and customized screen shots */
#include <SPI.h>
#include <Wire.h>
#include "FT_Platform.h"
#include "sdcard.h"
#include <avr/pgmspace.h>
//#include "FT_Hal_I2C.h"

#define SAMAPP_DELAY_BTW_APIS (1000)
#define SAMAPP_ENABLE_DELAY() Ft_Gpu_Hal_Sleep(SAMAPP_DELAY_BTW_APIS)
#define SAMAPP_ENABLE_DELAY_VALUE(x) Ft_Gpu_Hal_Sleep(x)
#define NOTOUCH		-32768
	
#define F16(s)        ((ft_int32_t)((s) * 65536))
#define SQ(v) (v*v)
#define MAX_MENUS 12
#define THUMBNAIL_ADDRESS (50*1024L)
#define MENU_POINTSIZE  5 // 16bit prec 


#define LOOPBACK_METHOD	 
//#define ANDROID_METHOD  
//#define WIN8_METHOD


//#define BACKGROUND_ANIMATION_1
//#define BACKGROUND_ANIMATION_2
#define BACKGROUND_ANIMATION_3
//#define BACKGROUND_ANIMATION_4
//#define BACKGROUND_ANIMATION_5
//#define BACKGROUND_ANIMATION_6

/* Global variables for display resolution to support various display panels */
/* Default is WQVGA - 480x272 */
ft_int16_t FT_DispWidth = 480;
ft_int16_t FT_DispHeight = 272;
ft_int16_t FT_DispHCycle =  548;
ft_int16_t FT_DispHOffset = 43;
ft_int16_t FT_DispHSync0 = 0;
ft_int16_t FT_DispHSync1 = 41;
ft_int16_t FT_DispVCycle = 292;
ft_int16_t FT_DispVOffset = 12;
ft_int16_t FT_DispVSync0 = 0;
ft_int16_t FT_DispVSync1 = 10;
ft_uint8_t FT_DispPCLK = 5;
ft_char8_t FT_DispSwizzle = 0;
ft_char8_t FT_DispPCLKPol = 1;
ft_char8_t FT_DispCSpread = 1;
ft_char8_t FT_DispDither = 1;

/* Global used for buffer optimization */
Ft_Gpu_Hal_Context_t host,*phost;

#ifdef FT900_PLATFORM
	FATFS FatFs;

#endif


const ft_uint8_t FT_DLCODE_BOOTUP[12] =
{
  255,255,255,2,//GPU instruction CLEAR_COLOR_RGB
  7,0,0,38, //GPU instruction CLEAR
  0,0,0,0,  //GPU instruction DISPLAY
};

ft_uint32_t Ft_CmdBuffer_Index;
ft_uint32_t Ft_DlBuffer_Index;

#ifdef BUFFER_OPTIMIZATION
ft_uint8_t  Ft_DlBuffer[FT_DL_SIZE];
ft_uint8_t  Ft_CmdBuffer[FT_CMD_FIFO_SIZE];
#endif


// deflated  icon
ft_void_t Ft_App_WrCoCmd_Buffer(Ft_Gpu_Hal_Context_t *phost,ft_uint32_t cmd)
{
#ifdef  BUFFER_OPTIMIZATION
   /* Copy the command instruction into buffer */
   ft_uint32_t *pBuffcmd;
   pBuffcmd =(ft_uint32_t*)&Ft_CmdBuffer[Ft_CmdBuffer_Index];
   *pBuffcmd = cmd;
#endif
#ifdef ARDUINO_PLATFORM
   Ft_Gpu_Hal_WrCmd32(phost,cmd);
#endif
#ifdef FT900_PLATFORM
   Ft_Gpu_Hal_WrCmd32(phost,cmd);
#endif
   /* Increment the command index */
   Ft_CmdBuffer_Index += FT_CMD_SIZE;
}

ft_void_t Ft_App_WrDlCmd_Buffer(Ft_Gpu_Hal_Context_t *phost,ft_uint32_t cmd)
{
#ifdef BUFFER_OPTIMIZATION
   /* Copy the command instruction into buffer */
   ft_uint32_t *pBuffcmd;
   pBuffcmd =(ft_uint32_t*)&Ft_DlBuffer[Ft_DlBuffer_Index];
   *pBuffcmd = cmd;
#endif

#ifdef ARDUINO_PLATFORM
   Ft_Gpu_Hal_Wr32(phost,(RAM_DL+Ft_DlBuffer_Index),cmd);
#endif
#ifdef FT900_PLATFORM
   Ft_Gpu_Hal_Wr32(phost,(RAM_DL+Ft_DlBuffer_Index),cmd);
#endif
   /* Increment the command index */
   Ft_DlBuffer_Index += FT_CMD_SIZE;
}

ft_void_t Ft_App_WrCoStr_Buffer(Ft_Gpu_Hal_Context_t *phost,const ft_char8_t *s)
{
#ifdef  BUFFER_OPTIMIZATION
  ft_uint16_t length = 0;
  length = strlen(s) + 1;//last for the null termination

  strcpy(&Ft_CmdBuffer[Ft_CmdBuffer_Index],s);

  /* increment the length and align it by 4 bytes */
  Ft_CmdBuffer_Index += ((length + 3) & ~3);
#endif
}

ft_void_t Ft_App_Flush_DL_Buffer(Ft_Gpu_Hal_Context_t *phost)
{
#ifdef  BUFFER_OPTIMIZATION
   if (Ft_DlBuffer_Index> 0)
     Ft_Gpu_Hal_WrMem(phost,RAM_DL,Ft_DlBuffer,Ft_DlBuffer_Index);
#endif
   Ft_DlBuffer_Index = 0;

}

ft_void_t Ft_App_Flush_Co_Buffer(Ft_Gpu_Hal_Context_t *phost)
{
#ifdef  BUFFER_OPTIMIZATION
   if (Ft_CmdBuffer_Index > 0)
     Ft_Gpu_Hal_WrCmdBuf(phost,Ft_CmdBuffer,Ft_CmdBuffer_Index);
#endif
   Ft_CmdBuffer_Index = 0;
}

/* API to give fadeout effect by changing the display PWM from 100 till 0 */
ft_void_t SAMAPP_fadeout()
{
   ft_int32_t i;

    for (i = 100; i >= 0; i -= 3)
    {
        Ft_Gpu_Hal_Wr8(phost,REG_PWM_DUTY,i);

        Ft_Gpu_Hal_Sleep(2);//sleep for 2 ms
    }
}

/* API to perform display fadein effect by changing the display PWM from 0 till 100 and finally 128 */
ft_void_t SAMAPP_fadein()
{
    ft_int32_t i;

    for (i = 0; i <=100 ; i += 3)
    {
        Ft_Gpu_Hal_Wr8(phost,REG_PWM_DUTY,i);
        Ft_Gpu_Hal_Sleep(2);//sleep for 2 ms
    }
    /* Finally make the PWM 100% */
    i = 128;
    Ft_Gpu_Hal_Wr8(phost,REG_PWM_DUTY,i);
}


/* API to check the status of previous DLSWAP and perform DLSWAP of new DL */
/* Check for the status of previous DLSWAP and if still not done wait for few ms and check again */
ft_void_t Ft_Gpu_DLSwap(ft_uint8_t DL_Swap_Type)
{
    ft_uint8_t Swap_Type = DLSWAP_FRAME,Swap_Done = DLSWAP_FRAME;

    if(DL_Swap_Type == DLSWAP_LINE)
    {
        Swap_Type = DLSWAP_LINE;
    }

    /* Perform a new DL swap */
    Ft_Gpu_Hal_Wr8(phost,REG_DLSWAP,Swap_Type);

    /* Wait till the swap is done */
    while(Swap_Done)
    {
        Swap_Done = Ft_Gpu_Hal_Rd8(phost,REG_DLSWAP);
        if(DLSWAP_DONE != Swap_Done)
        {
            Ft_Gpu_Hal_Sleep(10);//wait for 10ms
        }
    }
}


ft_void_t Ft_BootupConfig()
{
	Ft_Gpu_Hal_Powercycle(phost,FT_TRUE);

		/* Access address 0 to wake up the FT800 */
		Ft_Gpu_HostCommand(phost,FT_GPU_ACTIVE_M);
		Ft_Gpu_Hal_Sleep(20);

		/* Set the clk to external clock */
#if (!defined(ME800A_HV35R) && !defined(ME810A_HV35R))
		Ft_Gpu_HostCommand(phost,FT_GPU_EXTERNAL_OSC);
		Ft_Gpu_Hal_Sleep(10);
#endif

		{
			ft_uint8_t chipid;
			//Read Register ID to check if FT800 is ready.
			chipid = Ft_Gpu_Hal_Rd8(phost, REG_ID);
			while(chipid != 0x7C)
			{
				chipid = Ft_Gpu_Hal_Rd8(phost, REG_ID);
				ft_delay(100);
			}
	#if defined(MSVC_PLATFORM) || defined (FT900_PLATFORM)
			printf("VC1 register ID after wake up %x\n",chipid);
	#endif
	}
	/* Configuration of LCD display */
#ifdef DISPLAY_RESOLUTION_QVGA
	/* Values specific to QVGA LCD display */
	FT_DispWidth = 320;
	FT_DispHeight = 240;
	FT_DispHCycle =  408;
	FT_DispHOffset = 70;
	FT_DispHSync0 = 0;
	FT_DispHSync1 = 10;
	FT_DispVCycle = 263;
	FT_DispVOffset = 13;
	FT_DispVSync0 = 0;
	FT_DispVSync1 = 2;
	FT_DispPCLK = 8;
	FT_DispSwizzle = 2;
	FT_DispPCLKPol = 0;
	FT_DispCSpread = 1;
	FT_DispDither = 1;

#endif
#ifdef DISPLAY_RESOLUTION_WVGA
	/* Values specific to QVGA LCD display */
	FT_DispWidth = 800;
	FT_DispHeight = 480;
	FT_DispHCycle =  928;
	FT_DispHOffset = 88;
	FT_DispHSync0 = 0;
	FT_DispHSync1 = 48;
	FT_DispVCycle = 525;
	FT_DispVOffset = 32;
	FT_DispVSync0 = 0;
	FT_DispVSync1 = 3;
	FT_DispPCLK = 2;
	FT_DispSwizzle = 0;
	FT_DispPCLKPol = 1;
	FT_DispCSpread = 0;
	FT_DispDither = 1;
#endif
#ifdef DISPLAY_RESOLUTION_HVGA_PORTRAIT
	/* Values specific to HVGA LCD display */

	FT_DispWidth = 320;
	FT_DispHeight = 480;
	FT_DispHCycle =  400;
	FT_DispHOffset = 40;
	FT_DispHSync0 = 0;
	FT_DispHSync1 = 10;
	FT_DispVCycle = 500;
	FT_DispVOffset = 10;
	FT_DispVSync0 = 0;
	FT_DispVSync1 = 5;
	FT_DispPCLK = 4;
	FT_DispSwizzle = 2;
	FT_DispPCLKPol = 1;
	FT_DispCSpread = 1;
	FT_DispDither = 1;

#ifdef ME810A_HV35R
	FT_DispPCLK = 5;
#endif

#endif

#if (defined(ME800A_HV35R) || defined(ME810A_HV35R))
	/* After recognizing the type of chip, perform the trimming if necessary */
    Ft_Gpu_ClockTrimming(phost,LOW_FREQ_BOUND);
#endif

	Ft_Gpu_Hal_Wr16(phost, REG_HCYCLE, FT_DispHCycle);
	Ft_Gpu_Hal_Wr16(phost, REG_HOFFSET, FT_DispHOffset);
	Ft_Gpu_Hal_Wr16(phost, REG_HSYNC0, FT_DispHSync0);
	Ft_Gpu_Hal_Wr16(phost, REG_HSYNC1, FT_DispHSync1);
	Ft_Gpu_Hal_Wr16(phost, REG_VCYCLE, FT_DispVCycle);
	Ft_Gpu_Hal_Wr16(phost, REG_VOFFSET, FT_DispVOffset);
	Ft_Gpu_Hal_Wr16(phost, REG_VSYNC0, FT_DispVSync0);
	Ft_Gpu_Hal_Wr16(phost, REG_VSYNC1, FT_DispVSync1);
	Ft_Gpu_Hal_Wr8(phost, REG_SWIZZLE, FT_DispSwizzle);
	Ft_Gpu_Hal_Wr8(phost, REG_PCLK_POL, FT_DispPCLKPol);
	Ft_Gpu_Hal_Wr16(phost, REG_HSIZE, FT_DispWidth);
	Ft_Gpu_Hal_Wr16(phost, REG_VSIZE, FT_DispHeight);
	Ft_Gpu_Hal_Wr16(phost, REG_CSPREAD, FT_DispCSpread);
	Ft_Gpu_Hal_Wr16(phost, REG_DITHER, FT_DispDither);

#if (defined(FT_800_ENABLE) || defined(FT_810_ENABLE) ||defined(FT_812_ENABLE))
    /* Touch configuration - configure the resistance value to 1200 - this value is specific to customer requirement and derived by experiment */
    Ft_Gpu_Hal_Wr16(phost, REG_TOUCH_RZTHRESH,RESISTANCE_THRESHOLD);
#endif
    Ft_Gpu_Hal_Wr8(phost, REG_GPIO_DIR,0xff);
    Ft_Gpu_Hal_Wr8(phost, REG_GPIO,0xff);


    /*It is optional to clear the screen here*/
    Ft_Gpu_Hal_WrMem(phost, RAM_DL,(ft_uint8_t *)FT_DLCODE_BOOTUP,sizeof(FT_DLCODE_BOOTUP));
    Ft_Gpu_Hal_Wr8(phost, REG_DLSWAP,DLSWAP_FRAME);


    Ft_Gpu_Hal_Wr8(phost, REG_PCLK,FT_DispPCLK);//after this display is visible on the LCD


#ifdef ENABLE_ILI9488_HVGA_PORTRAIT
	/* to cross check reset pin */
	Ft_Gpu_Hal_Wr8(phost, REG_GPIO,0xff);
	ft_delay(120);
	Ft_Gpu_Hal_Wr8(phost, REG_GPIO,0x7f);
	ft_delay(120);
	Ft_Gpu_Hal_Wr8(phost, REG_GPIO,0xff);

	ILI9488_Bootup();

	/* Reconfigure the SPI */
#ifdef FT900_PLATFORM
	printf("after ILI9488 bootup \n");
	//spi
	// Initialize SPIM HW
	sys_enable(sys_device_spi_master);
	gpio_function(27, pad_spim_sck); /* GPIO27 to SPIM_CLK */
	gpio_function(28, pad_spim_ss0); /* GPIO28 as CS */
	gpio_function(29, pad_spim_mosi); /* GPIO29 to SPIM_MOSI */
	gpio_function(30, pad_spim_miso); /* GPIO30 to SPIM_MISO */

	gpio_write(28, 1);
	spi_init(SPIM, spi_dir_master, spi_mode_0, 4);
#endif

#endif

	/* make the spi to quad mode - addition 2 bytes for silicon */
#ifdef FT_81X_ENABLE
	/* api to set quad and numbe of dummy bytes */
#ifdef ENABLE_SPI_QUAD
	Ft_Gpu_Hal_SetSPI(phost,FT_GPU_SPI_QUAD_CHANNEL,FT_GPU_SPI_TWODUMMY);
#elif ENABLE_SPI_DUAL
	Ft_Gpu_Hal_SetSPI(phost,FT_GPU_SPI_DUAL_CHANNEL,FT_GPU_SPI_TWODUMMY);
#else
	Ft_Gpu_Hal_SetSPI(phost,FT_GPU_SPI_SINGLE_CHANNEL,FT_GPU_SPI_ONEDUMMY);
#endif

#endif


#ifdef FT900_PLATFORM
    /* Change clock frequency to 25mhz */
	spi_init(SPIM, spi_dir_master, spi_mode_0, 4);

#if (defined(ENABLE_SPI_QUAD))
    /* Initialize IO2 and IO3 pad/pin for dual and quad settings */
    gpio_function(31, pad_spim_io2);
    gpio_function(32, pad_spim_io3);
    gpio_write(31, 1);
    gpio_write(32, 1);
#endif
	/* Enable FIFO of QSPI */
	spi_option(SPIM,spi_option_fifo_size,64);
	spi_option(SPIM,spi_option_fifo,1);
	spi_option(SPIM,spi_option_fifo_receive_trigger,1);
#endif

#ifdef ENABLE_SPI_QUAD
#ifdef FT900_PLATFORM
	spi_option(SPIM,spi_option_bus_width,4);
#endif
#elif ENABLE_SPI_DUAL	
#ifdef FT900_PLATFORM
	spi_option(SPIM,spi_option_bus_width,2);
#endif
#else
#ifdef FT900_PLATFORM
	spi_option(SPIM,spi_option_bus_width,1);
#endif

	
#endif


	phost->ft_cmd_fifo_wp = Ft_Gpu_Hal_Rd16(phost,REG_CMD_WRITE);

}


/*****************************************************************************/
/* Example code to display few points at various offsets with various colors */





FT_PROGMEM ft_prog_uint16_t sintab[] = {
0, 402, 804, 1206, 1607, 2009, 2410, 2811, 3211, 3611, 4011, 4409, 4807, 5205, 5601, 5997, 6392,
6786, 7179, 7571, 7961, 8351, 8739, 9126, 9511, 9895, 10278, 10659, 11038, 11416, 11792, 12166, 12539,
12909, 13278, 13645, 14009, 14372, 14732, 15090, 15446, 15799, 16150, 16499, 16845, 17189, 17530, 17868,
18204, 18537, 18867, 19194, 19519, 19840, 20159, 20474, 20787, 21096, 21402, 21705, 22004, 22301, 22594,
22883, 23169, 23452, 23731, 24006, 24278, 24546, 24811, 25072, 25329, 25582, 25831, 26077, 26318, 26556, 26789,
27019, 27244, 27466, 27683, 27896, 28105, 28309, 28510, 28706, 28897, 29085, 29268, 29446, 29621, 29790, 29955,
30116, 30272, 30424, 30571, 30713, 30851, 30984, 31113, 31236, 31356, 31470, 31580, 31684, 31785, 31880, 31970,
32056, 32137, 32213, 32284, 32350, 32412, 32468, 32520, 32567, 32609, 32646, 32678, 32705, 32727, 32744, 32757,
32764, 32767, 32764};

static PROGMEM prog_uchar home_star_icon[] = {0x78,0x9C,0xE5,0x94,0xBF,0x4E,0xC2,0x40,0x1C,0xC7,0x7F,0x2D,0x04,0x8B,0x20,0x45,0x76,0x14,0x67,0xA3,0xF1,0x0D,0x64,0x75,0xD2,0xD5,0x09,0x27,0x17,0x13,0xE1,0x0D,0xE4,0x0D,0x78,0x04,0x98,0x5D,0x30,0x26,0x0E,0x4A,0xA2,0x3E,0x82,0x0E,0x8E,0x82,0xC1,0x38,0x62,0x51,0x0C,0x0A,0x42,0x7F,0xDE,0xB5,0x77,0xB4,0x77,0x17,0x28,0x21,0x26,0x46,0xFD,0x26,0xCD,0xE5,0xD3,0x7C,0xFB,0xBB,0xFB,0xFD,0xB9,0x02,0xCC,0xA4,0xE8,0x99,0x80,0x61,0xC4,0x8A,0x9F,0xCB,0x6F,0x31,0x3B,0xE3,0x61,0x7A,0x98,0x84,0x7C,0x37,0xF6,0xFC,0xC8,0xDD,0x45,0x00,0xDD,0xBA,0xC4,0x77,0xE6,0xEE,0x40,0xEC,0x0E,0xE6,0x91,0xF1,0xD2,0x00,0x42,0x34,0x5E,0xCE,0xE5,0x08,0x16,0xA0,0x84,0x68,0x67,0xB4,0x86,0xC3,0xD5,0x26,0x2C,0x20,0x51,0x17,0xA2,0xB8,0x03,0xB0,0xFE,0x49,0xDD,0x54,0x15,0xD8,0xEE,0x73,0x37,0x95,0x9D,0xD4,0x1A,0xB7,0xA5,0x26,0xC4,0x91,0xA9,0x0B,0x06,0xEE,0x72,0xB7,0xFB,0xC5,0x16,0x80,0xE9,0xF1,0x07,0x8D,0x3F,0x15,0x5F,0x1C,0x0B,0xFC,0x0A,0x90,0xF0,0xF3,0x09,0xA9,0x90,0xC4,0xC6,0x37,0xB0,0x93,0xBF,0xE1,0x71,0xDB,0xA9,0xD7,0x41,0xAD,0x46,0xEA,0x19,0xA9,0xD5,0xCE,0x93,0xB3,0x35,0x73,0x0A,0x69,0x59,0x91,0xC3,0x0F,0x22,0x1B,0x1D,0x91,0x13,0x3D,0x91,0x73,0x43,0xF1,0x6C,0x55,0xDA,0x3A,0x4F,0xBA,0x25,0xCE,0x4F,0x04,0xF1,0xC5,0xCF,0x71,0xDA,0x3C,0xD7,0xB9,0xB2,0x48,0xB4,0x89,0x38,0x20,0x4B,0x2A,0x95,0x0C,0xD5,0xEF,0x5B,0xAD,0x96,0x45,0x8A,0x41,0x96,0x7A,0x1F,0x60,0x0D,0x7D,0x22,0x75,0x82,0x2B,0x0F,0xFB,0xCE,0x51,0x3D,0x2E,0x3A,0x21,0xF3,0x1C,0xD9,0x38,0x86,0x2C,0xC6,0x05,0xB6,0x7B,0x9A,0x8F,0x0F,0x97,0x1B,0x72,0x6F,0x1C,0xEB,0xAE,0xFF,0xDA,0x97,0x0D,0xBA,0x43,0x32,0xCA,0x66,0x34,0x3D,0x54,0xCB,0x24,0x9B,0x43,0xF2,0x70,0x3E,0x42,0xBB,0xA0,0x95,0x11,0x37,0x46,0xE1,0x4F,0x49,0xC5,0x1B,0xFC,0x3C,0x3A,0x3E,0xD1,0x65,0x0E,0x6F,0x58,0xF8,0x9E,0x5B,0xDB,0x55,0xB6,0x41,0x34,0xCB,0xBE,0xDB,0x87,0x5F,0xA9,0xD1,0x85,0x6B,0xB3,0x17,0x9C,0x61,0x0C,0x9B,0xA2,0x5D,0x61,0x10,0xED,0x2A,0x9B,0xA2,0x5D,0x61,0x10,0xED,0x2A,0x9B,0xA2,0x5D,0x61,0x10,0xED,0x2A,0x9B,0xED,0xC9,0xFC,0xDF,0x14,0x54,0x8F,0x80,0x7A,0x06,0xF5,0x23,0xA0,0x9F,0x41,0xF3,0x10,0x30,0x4F,0x41,0xF3,0x18,0x30,0xCF,0xCA,0xFC,0xFF,0x35,0xC9,0x79,0xC9,0x89,0xFA,0x33,0xD7,0x1D,0xF6,0x5E,0x84,0x5C,0x56,0x6E,0xA7,0xDA,0x1E,0xF9,0xFA,0xAB,0xF5,0x97,0xFF,0x2F,0xED,0x89,0x7E,0x29,0x9E,0xB4,0x9F,0x74,0x1E,0x69,0xDA,0xA4,0x9F,0x81,0x94,0xEF,0x4F,0xF6,0xF9,0x0B,0xF4,0x65,0x51,0x08};


prog_char info_1[] PROGMEM =  "FT800 MainMenu Application";
prog_char info_2[] PROGMEM =  "APP to demonstrate interactive menus,";
prog_char info_3[] PROGMEM =  "using Jpeg decode,";
prog_char info_4[] PROGMEM =  "Rectangle & Points.";

PROGMEM const char *information[] = 	{   
											  info_1,
											  info_2,
											  info_3,
											  info_4,
										};
                           
static ft_uint8_t sd_present =0;


Reader imfile;

// API used to play the KEYsound
void Ft_Play_Sound(ft_uint8_t sound,ft_uint8_t vol,ft_uint8_t midi)
{
  ft_uint16_t val = (midi << 8) | sound; 
  Ft_Gpu_Hal_Wr8(phost,REG_SOUND,val);
  Ft_Gpu_Hal_Wr8(phost,REG_PLAY,1); 
  return;
}

static struct {
  signed short dragprev;
  int vel;      // velocity
  long base;    // screen x coordinate, in 1/16ths pixel
  long limit;
} scroller;

/* Api to Intialise the scroller*/
static void scroller_init(ft_uint32_t limit)
{
  scroller.dragprev = -32768;
  scroller.vel = 0;      // velocity
  scroller.base = 0;     // screen x coordinate, in 1/16ths pixel
  scroller.limit = limit;
}
/* Api used to scroll the screen horizontally*/
static void scroller_run()
{
  signed short sx;
  static ft_uint16_t _delay = 0;
#ifdef FT_801_ENABLE  
  static ft_uint32_t prev_time = 0;
  ft_uint32_t time  = millis();
  if(prev_time!=0)
  _delay += (time-prev_time); 
  prev_time = time;
  if(_delay<30)
  {
	scroller.base += scroller.vel;
	scroller.base = max(0, min(scroller.base, scroller.limit));
	return; 
  }		
#endif 
  sx = Ft_Gpu_Hal_Rd16(phost,REG_TOUCH_SCREEN_XY + 2);
  if ((sx != -32768) & (scroller.dragprev != -32768)) {
    scroller.vel = (scroller.dragprev - sx) << 4;
  } else {
    int change = max(1, abs(scroller.vel) >> 5);
    if (scroller.vel < 0)
      scroller.vel += change;
    if (scroller.vel > 0)
      scroller.vel -= change;
  }
  scroller.dragprev = sx;
  scroller.base += scroller.vel;
  scroller.base = max(0, min(scroller.base, scroller.limit));
  _delay = 0;
}
/********API to return the assigned TAG value when penup,for the primitives/widgets******/
ft_uint8_t keypressed = 0;
ft_uint8_t keyin_cts = 0;
#define KEYIN_COUNTS	10
ft_uint8_t Read_keys()
{	
	static ft_uint8_t temp_key = 0,prev_keyin = 0; 
	keypressed = Ft_Gpu_Hal_Rd8(phost,REG_TOUCH_TAG);
	if(keypressed!=NULL && temp_key!=keypressed)
	temp_key = keypressed;
	if(keypressed==NULL)
	{
		if(temp_key!=NULL)
		{		
			prev_keyin = temp_key;
			temp_key = NULL;
			return prev_keyin;
		}
		temp_key = 	keypressed;	
	}	
	return 0;
}



/* Load the icons of the menu to GRAM*/ 
void Load_Thumbnails()
{
  #ifdef MSVC_PLATFORM	 	
  char *apps[] = {"1.jpg","2.jpg","3.jpg","4.jpg","5.jpg","6.jpg","7.jpg","8.jpg","9.jpg","10.jpg","11.jpg","12.jpg"};
  #endif
  #ifdef ARDUINO_PLATFORM 
  String stringOne; 
  #endif
  
  if(sd_present==0)
  {
    for(int i=0;i<12;i++)
    {
      Ft_Gpu_CoCmd_Dlstart(phost); 
      Ft_Gpu_CoCmd_MemSet(phost,(i*10000)+THUMBNAIL_ADDRESS,ft_random(10*1024L),200*50);
      Ft_Gpu_CoCmd_Swap(phost);
      Ft_App_Flush_Co_Buffer(phost);
      Ft_Gpu_Hal_WaitCmdfifo_empty(phost);
    }  
    return;
  }
  int image_no = 1;
  do
  {	
	stringOne = 0;
	stringOne+=image_no;
	stringOne+=".jpg";	
	char apps[stringOne.length()+1];
	stringOne.toCharArray(apps,stringOne.length()+1);
	byte ifile = imfile.openfile(apps);
    ft_int16_t fsize  = imfile.size;
    if(fsize==NULL)
    {
    /*
		Ft_Gpu_Hal_WrCmd32(phost,CMD_MEMSET);
		Ft_Gpu_Hal_WrCmd32(phost,(image_no-1)*10*1024L);
		Ft_Gpu_Hal_WrCmd32(phost,ft_random(150));
		Ft_Gpu_Hal_WrCmd32(phost,ft_random(10*1024L));
	*/
    }
    else
    {
		Ft_Gpu_Hal_WrCmd32(phost,CMD_LOADIMAGE);
		Ft_Gpu_Hal_WrCmd32(phost,THUMBNAIL_ADDRESS+((image_no-1)*10000));
		Ft_Gpu_Hal_WrCmd32(phost,0);
    }
	ft_uint8_t *buff = (ft_uint8_t*)malloc(512);  
    while(fsize > 0)
    {
      ft_int16_t wsize = fsize > 512L ? 512L:fsize;
      imfile.readsector(buff);
      fsize -= wsize;
      Ft_Gpu_Hal_WrCmdBuf(phost,buff,wsize);				// copy data continuously into command memory 
    }
	free(buff);
    image_no++;
  }while(image_no<=MAX_MENUS);
  Ft_DlBuffer_Index = 0;		
  Ft_App_WrDlCmd_Buffer(phost,BITMAP_HANDLE(0));	
  Ft_App_WrDlCmd_Buffer(phost,BITMAP_SOURCE(THUMBNAIL_ADDRESS));	
  Ft_App_WrDlCmd_Buffer(phost,BITMAP_LAYOUT(RGB565,100*2,50));	
  Ft_App_WrDlCmd_Buffer(phost,BITMAP_SIZE(NEAREST, BORDER, BORDER, 100, 50));	
  Ft_App_WrDlCmd_Buffer(phost,DISPLAY());
  Ft_App_Flush_DL_Buffer(phost);
  Ft_Gpu_DLSwap(DLSWAP_FRAME);
}

/***********************API used to SET the ICON******************************************/
/*Refer the code flow in the flowchart availble in the Application Note */


ft_void_t home_setup()
{
  /*Icon  file is deflated use J1 Command to inflate the file and write into the GRAM*/
  Ft_Gpu_Hal_WrCmd32(phost,CMD_INFLATE);
  Ft_Gpu_Hal_WrCmd32(phost,250*1024L);
  Ft_Gpu_Hal_WrCmdBufFromFlash(phost,home_star_icon,sizeof(home_star_icon)); 
  /*Set the Bitmap properties for the ICONS*/
  Ft_Gpu_CoCmd_Dlstart(phost);        // start
  Ft_App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));
  Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255, 255, 255));
  Ft_App_WrCoCmd_Buffer(phost,BITMAP_HANDLE(14));    // handle for background stars
  Ft_App_WrCoCmd_Buffer(phost,BITMAP_SOURCE(250*1024L));      // Starting address in gram
  Ft_App_WrCoCmd_Buffer(phost,BITMAP_LAYOUT(L4, 16, 32));  // format 
  Ft_App_WrCoCmd_Buffer(phost,BITMAP_SIZE(NEAREST, BORDER, BORDER, 32, 32  ));
  Ft_App_WrCoCmd_Buffer(phost,DISPLAY());
  Ft_Gpu_CoCmd_Swap(phost);
  Ft_App_Flush_Co_Buffer(phost);
  Ft_Gpu_Hal_WaitCmdfifo_empty(phost);
}
//==============================================Information Screen =======================================
ft_void_t Info()
{
  ft_uint16_t dloffset = 0,z;
 // Touch Screen Calibration
  Ft_Gpu_CoCmd_Dlstart(phost); 
  Ft_App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));
  Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
  Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,FT_DispHeight/2,26,OPT_CENTERX|OPT_CENTERY,"Please tap on a dot");
  Ft_Gpu_CoCmd_Calibrate(phost,0);
  Ft_App_WrCoCmd_Buffer(phost,DISPLAY());
  Ft_Gpu_CoCmd_Swap(phost);
  Ft_App_Flush_Co_Buffer(phost);
  Ft_Gpu_Hal_WaitCmdfifo_empty(phost);
// Ftdi Logo animation 
  Ft_Gpu_CoCmd_Logo(phost);
  Ft_App_Flush_Co_Buffer(phost);
  Ft_Gpu_Hal_WaitCmdfifo_empty(phost);
  while(0!=Ft_Gpu_Hal_Rd16(phost,REG_CMD_READ)); 
  dloffset = Ft_Gpu_Hal_Rd16(phost,REG_CMD_DL);
  dloffset -=4;										/*ignore the display command*/
  //Copy the Displaylist from DL RAM to GRAM
  Ft_Gpu_Hal_WrCmd32(phost,CMD_MEMCPY);
  Ft_Gpu_Hal_WrCmd32(phost,100000L);
  Ft_Gpu_Hal_WrCmd32(phost,RAM_DL);
  Ft_Gpu_Hal_WrCmd32(phost,dloffset);
  ft_uint8_t key_in = 0;
  do
  {	
    Ft_Gpu_CoCmd_Dlstart(phost);   
    Ft_Gpu_CoCmd_Append(phost,100000L,dloffset);
    //Reset the BITMAP properties used during Logo animation
    Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_A(256));
    Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_A(256));
    Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_B(0));
    Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_C(0));
    Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_D(0));
    Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_E(256));
    Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_F(0));  
    Ft_App_WrCoCmd_Buffer(phost,SAVE_CONTEXT());	
    //Display the information with transparent Logo using Edge Strip 
    Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(219,180,150));
    Ft_App_WrCoCmd_Buffer(phost,COLOR_A(220));
    Ft_App_WrCoCmd_Buffer(phost,BEGIN(EDGE_STRIP_A));
    Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(0,FT_DispHeight*16));
    Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(FT_DispWidth*16,FT_DispHeight*16));
    Ft_App_WrCoCmd_Buffer(phost,COLOR_A(255));
    Ft_App_WrCoCmd_Buffer(phost,RESTORE_CONTEXT());	
    Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,0,0));
   // INFORMATION 
    char buffer[40];
	strcpy_P(buffer, (char*)pgm_read_word(&(information[0])));
    Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,20,28,OPT_CENTERX|OPT_CENTERY,buffer);
	strcpy_P(buffer, (char*)pgm_read_word(&(information[1])));
    Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,60,26,OPT_CENTERX|OPT_CENTERY,buffer);
	strcpy_P(buffer, (char*)pgm_read_word(&(information[2])));
    Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,90,26,OPT_CENTERX|OPT_CENTERY,buffer);  
	strcpy_P(buffer, (char*)pgm_read_word(&(information[3])));
    Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,120,26,OPT_CENTERX|OPT_CENTERY,buffer);  
    Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,FT_DispHeight-30,26,OPT_CENTERX|OPT_CENTERY,"Click to play");
    //Check if the Play key and change the color
    if(keypressed!='P')
    Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
    else
    Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(100,100,100));
    Ft_App_WrCoCmd_Buffer(phost,BEGIN(FTPOINTS));   
    Ft_App_WrCoCmd_Buffer(phost,POINT_SIZE(20*16));
    Ft_App_WrCoCmd_Buffer(phost,TAG('P'));
    Ft_App_WrCoCmd_Buffer(phost,VERTEX2F((FT_DispWidth/2)*16,(FT_DispHeight-60)*16));
    Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(180,35,35));
    Ft_App_WrCoCmd_Buffer(phost,BEGIN(BITMAPS));
    Ft_App_WrCoCmd_Buffer(phost,VERTEX2II((FT_DispWidth/2)-14,(FT_DispHeight-75),14,4));
    Ft_App_WrCoCmd_Buffer(phost,DISPLAY());
    Ft_Gpu_CoCmd_Swap(phost);
    Ft_App_Flush_Co_Buffer(phost);
    Ft_Gpu_Hal_WaitCmdfifo_empty(phost);
  }while(Read_keys()!='P');
  /* wait until Play key is not pressed*/
}

static ft_uint8_t istouch()
{
  return !(Ft_Gpu_Hal_Rd16(phost,REG_TOUCH_RAW_XY) & 0x8000);
}

void Sine_wave(ft_uint8_t amp,ft_uint16_t address,ft_uint16_t yoffset)
{
  ft_uint16_t x = 0,y=0; 

  for(x=0;x<FT_DispWidth+100;x+=10)
  {		
	y = (yoffset) + ((ft_int32_t)amp*qsin(-65536*x/(25*10))/65536);
	Ft_Gpu_Hal_Wr32(phost,address+(x/10)*4,VERTEX2F(x*16,y*16));	
  }	
}

float linear(float p1,float p2,float t,ft_uint16_t rate)
{
	float st  = (float)t/rate;
	return p1+(st*(p2-p1));
}
float acceleration(float p1,float p2,ft_uint16_t t,ft_uint16_t rate)
{
	float dst  = (float)t/rate;
	float st = SQ(dst);
	return p1+(st*(p2-p1));
}

float deceleration(float p1,float p2,ft_uint16_t t,ft_uint16_t rate)
{
	float st,dst  = (float)t/rate;
	dst = 1-dst;
	st = 1-SQ(dst);
	return p1+(st*(p2-p1));
}

ft_int16_t qsin(ft_uint16_t a)
{
  ft_uint8_t f;
  ft_int16_t s0,s1;

  if (a & 32768)
    return -qsin(a & 32767);
  if (a & 16384)
      a = 32768 - a;
  f = a & 127;
  s0 = ft_pgm_read_word(sintab + (a >> 7));
  s1 = ft_pgm_read_word(sintab + (a >> 7) + 1);
  return (s0 + ((ft_int32_t)f * (s1 - s0) >> 7));
}

/* cos funtion */
ft_int16_t qcos(ft_uint16_t a)
{
  return (qsin(a + 16384));
}

static void polar(ft_int32_t r, float th,ft_uint16_t ox,ft_uint16_t oy)
{
  ft_int32_t x, y;
  th = (th * 32768L / 180);
  x = (16 * ox + (((long)r * qsin(th)) >> 11) + 16);
  y = (16 * oy - (((long)r * qcos(th)) >> 11));
  Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(x,y));
}


ft_int16_t ftsize;
#define rotation_rate 2


void Load_file2gram(ft_uint32_t add,ft_uint8_t sectors,Reader &r)
{
  ft_uint8_t pbuff[512];
  while(ftsize>0)
  {
    r.readsector(pbuff);
    Ft_Gpu_Hal_WrMem(phost,add,pbuff,512L);
    add+=512;
    ftsize-=512;
  }  
}

#ifdef BACKGROUND_ANIMATION_1

ft_uint16_t xoffset_array[30],dx_pts[20];
ft_uint8_t yoffset_array[30],bitmap_handle[30],dy_pts[20];
ft_uint8_t rate_cts[30],iteration_cts[30];

void Backgroundanimation_1()
{
	static ft_uint16_t bg_cts  = 0,fg = 0,wave_cts = 490;
	static ft_uint8_t init = 0,_cell = 0,VEL;
	static ft_uint8_t cts = 0;
	ft_int16_t i = 0,j=0,xoff,yoff;
	float _xoff = 0;
	wave_cts = 490;
	if(cts>=5){cts = 0;if(_cell>0)_cell--; else _cell = 7;}cts++;
	_cell = 0;
	if(wave_cts<580)wave_cts+=10; else wave_cts = 0;
	if(istouch()) VEL = 2; else VEL = 1;

	if(!init)
	{
      	init = 1;
		Sine_wave(15,RAM_G,FT_DispHeight/2);		
		Sine_wave(12,RAM_G+232,16+(FT_DispHeight/2));
		Sine_wave(9,RAM_G+2*232,32+(FT_DispHeight/2));
		Sine_wave(6,RAM_G+3*232,48+(FT_DispHeight/2));
		for(i=0;i<30;i++)
		{		
			yoffset_array[i] = ft_random(255);
			bitmap_handle[i] = 4+ft_random(4);
			rate_cts[i] = 100+ft_random(155);
			iteration_cts[i] = ft_random(200);
		}

		byte file = imfile.openfile("nts1.raw");
		ftsize = imfile.size; 
	   // Serial.println(ftsize,DEC);                
		Load_file2gram(220*1024L,1,imfile);
		
		file = imfile.openfile("nts2.raw");               
		ftsize = imfile.size;
	   // Serial.println(ftsize,DEC);
		Load_file2gram(222*1024L,1,imfile);
		
		file = imfile.openfile("nts3.raw");               
		ftsize = imfile.size;               
		Load_file2gram(256506L,1,imfile);
		
		file = imfile.openfile("nts4.raw");               
		ftsize = imfile.size;
		Load_file2gram(257306L,1,imfile);
		
		file = imfile.openfile("hline.raw");               
		ftsize = imfile.size;
		Load_file2gram(255*1024L,1,imfile);
                
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_HANDLE(4));	
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_SOURCE(220*1024L));	
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_LAYOUT(L4,10,50));	
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_SIZE(BILINEAR, BORDER, BORDER, 40, 100));		
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_HANDLE(5));	
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_SOURCE(222*1024L));	
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_LAYOUT(L4,25,60));	
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_SIZE(BILINEAR, BORDER, BORDER, 100, 120));		
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_HANDLE(6));	
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_SOURCE(256506L));	
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_LAYOUT(L4,10,40));	
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_SIZE(BILINEAR, BORDER, BORDER, 40, 80));			
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_HANDLE(7));	
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_SOURCE(257306L));	
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_LAYOUT(L4,10,24));	
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_SIZE(BILINEAR, BORDER, BORDER, 40, 48));		
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_HANDLE(8));	
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_SOURCE(255*1024L));	
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_LAYOUT(L8,512,1));	
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_SIZE(NEAREST, REPEAT, REPEAT, 512, 272));		
	}
//	
    Ft_App_WrCoCmd_Buffer(phost,SAVE_CONTEXT());
	Ft_App_WrCoCmd_Buffer(phost,CLEAR_COLOR_RGB(30,30,30));
	Ft_App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));
	Ft_App_WrCoCmd_Buffer(phost,COLOR_A(170));
	
	
	Ft_App_WrCoCmd_Buffer(phost,BEGIN(BITMAPS));
	Ft_App_WrCoCmd_Buffer(phost,BITMAP_HANDLE(8));	
	Ft_App_WrCoCmd_Buffer(phost,VERTEX2F((ft_int16_t)0,0));	
	
	Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(30,30,30));
	Ft_App_WrCoCmd_Buffer(phost,COLOR_A(255));
	


	for(j=0;j<3;j++)
	{
	  switch(j)
	  {
		case 0:
		  Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_A(384));
		  Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_E(384));
		break;

		case 1:
		  Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_A(256));
		  Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_E(256));
		break;

		case 2:			
		  Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_A(200));
		  Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_E(200));
		break;
	  }
	  for(i = 0;i<10;i++)
	  {
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_HANDLE(bitmap_handle[j*10+i]));
		if(bitmap_handle[j*10+i]==5)
		Ft_App_WrCoCmd_Buffer(phost,CELL(_cell));
		else 
		{
		  switch(j)
		  {
			case 0:
			Ft_App_WrCoCmd_Buffer(phost,CELL(0));
			break;

			case 1:
			Ft_App_WrCoCmd_Buffer(phost,CELL(0));
			break;

			case 2:
			Ft_App_WrCoCmd_Buffer(phost,CELL(1));				
			break;
		  }
		}
		xoff = linear(FT_DispWidth,-50,iteration_cts[j*10+i],rate_cts[j*10+i]);
		yoff = linear(FT_DispHeight/2,yoffset_array[j*10+i],iteration_cts[j*10+i],rate_cts[j*10+i]);
		Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(xoff*16,yoff*16));
	  }
	}
	for(i = 0; i<30 ; i++)
	{
	    if(iteration_cts[i]==0)
	    {
			yoffset_array[i] = ft_random(255);
			bitmap_handle[i] = 4+ft_random(4);
	    }
	    if(iteration_cts[i]<rate_cts[i])iteration_cts[i]+=VEL; else{ iteration_cts[i] = 0;}
	}
	
	wave_cts = 490;	
	Ft_App_WrCoCmd_Buffer(phost,LINE_WIDTH(1*16));
	Ft_App_WrCoCmd_Buffer(phost,COLOR_A(255));
	
	Ft_App_WrCoCmd_Buffer(phost,BEGIN(LINE_STRIP));
	Ft_Gpu_CoCmd_Append(phost,RAM_G,(wave_cts/10)*4);    
	Ft_App_WrCoCmd_Buffer(phost,END());
	
	Ft_App_WrCoCmd_Buffer(phost,BEGIN(LINE_STRIP));
	Ft_Gpu_CoCmd_Append(phost,RAM_G+232,(wave_cts/10)*4);
	Ft_App_WrCoCmd_Buffer(phost,END());
	
	Ft_App_WrCoCmd_Buffer(phost,BEGIN(LINE_STRIP));
	Ft_Gpu_CoCmd_Append(phost,RAM_G+2*232,(wave_cts/10)*4);
	Ft_App_WrCoCmd_Buffer(phost,END());
	
	Ft_App_WrCoCmd_Buffer(phost,BEGIN(LINE_STRIP));
	Ft_Gpu_CoCmd_Append(phost,RAM_G+3*232,(wave_cts/10)*4);
	Ft_App_WrCoCmd_Buffer(phost,END());
	
	Ft_App_WrCoCmd_Buffer(phost,RESTORE_CONTEXT());
}
#endif

#ifdef BACKGROUND_ANIMATION_2

ft_uint16_t point_size[20];
ft_uint16_t xoffset_array[20],yoffset_array[20],dx_pts[20],dy_pts[20];
ft_uint8_t color[20];

void Backgroundanimation_2()
{
	static ft_uint8_t init = 0;	
	ft_uint8_t ptclrarray[6][3] = {
		{0xb9,0xba,0xde},
		{0x0c,0x61,0xb7},
		{0x01,0x18,0x4e},
		{0xbf,0x25,0xbd},
		{0x29,0x07,0x3a},
		{0xc9,0x61,0x22}};
	ft_int32_t i,ptradius,colorindex;
	static ft_uint8_t fg = 0;
	float vel = 2;
	static float t=0,t1=0;	
	ft_int32_t xoffset,yoffset,x1offset,y1offset,rate = 2000;
	ft_uint8_t pts;
	if(!init)
	{
		init = 1;
		for(pts=0;pts<20;pts++)
		{
			point_size[pts] = 136 *16 +ft_random(375*16); 
			xoffset_array[pts] = ft_random(512)*16;
			yoffset_array[pts] = ft_random(512)*16;
			color[pts] = ft_random(5);
			dx_pts[pts] = 240*16+ft_random(240*16);
			dy_pts[pts] = 130*16+ft_random(142*16);	
		}
	}
	if(istouch())	
	{ t1 = 0; vel  = 5; } else {
		vel = linear(5,2,t1,100); 
		if(t1<100) t1++; }
	Ft_App_WrCoCmd_Buffer(phost,SAVE_CONTEXT());
	Ft_App_WrCoCmd_Buffer(phost,COLOR_MASK(1,1,1,1));
	Ft_App_WrCoCmd_Buffer(phost,COLOR_A(255));


	//draw 20 points with various radius and additive blending
	Ft_App_WrCoCmd_Buffer(phost,BEGIN(FTPOINTS));
	Ft_App_WrCoCmd_Buffer(phost,BLEND_FUNC(SRC_ALPHA,ONE));
	Ft_App_WrCoCmd_Buffer(phost,COLOR_MASK(1,1,1,0));
	Ft_App_WrCoCmd_Buffer(phost,COLOR_A(50));

	/* compute points on top */
	for(i=0;i<5;i++)
	{
		ptradius = point_size[i]; 
		colorindex = color[i+0];
		Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(ptclrarray[colorindex][0],ptclrarray[colorindex][1],ptclrarray[colorindex][2]));
		yoffset = linear(dy_pts[i+0],yoffset_array[i+0],t,1000);
		
		xoffset = xoffset_array[i+0];
		Ft_App_WrCoCmd_Buffer(phost,POINT_SIZE(ptradius));
		Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(xoffset,yoffset));
	}
	/* compute points on right */
	for(i=0;i<5;i++)
	{
		ptradius = point_size[5+i];
		colorindex = color[i+5];	
		Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(ptclrarray[colorindex][0],ptclrarray[colorindex][1],ptclrarray[colorindex][2]));				
		yoffset = linear(dy_pts[i+5],yoffset_array[i+5],t,1000);			
	
		yoffset = yoffset_array[i+5];		
		Ft_App_WrCoCmd_Buffer(phost,POINT_SIZE(ptradius));
		Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(xoffset,yoffset));
	}
	///* compute points on left */
	for(i=0;i<5;i++)
	{
		ptradius = point_size[10+i];
		colorindex = color[i+10];		
		
		Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(ptclrarray[colorindex][0],ptclrarray[colorindex][1],ptclrarray[colorindex][2]));		
		xoffset = linear(dx_pts[i+0],xoffset_array[i+0],t,1000);
		
		yoffset = yoffset_array[i+5];
		Ft_App_WrCoCmd_Buffer(phost,POINT_SIZE(ptradius));
		Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(xoffset,yoffset));
	}
	/* compute points on bottom */
	for(i=0;i<5;i++)
	{
		ptradius = point_size[15+i];
		colorindex = color[i+15];		
		Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(ptclrarray[colorindex][0],ptclrarray[colorindex][1],ptclrarray[colorindex][2]));
		yoffset = linear(dy_pts[i+15],yoffset_array[i+15],t,1000);
		xoffset = xoffset_array[i+15];		
		Ft_App_WrCoCmd_Buffer(phost,POINT_SIZE(ptradius));
		Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(xoffset,yoffset));
	}
	
	//draw additive blend lines diagonally
	Ft_App_WrCoCmd_Buffer(phost,COLOR_A(100));
	Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(32,32,32));
	Ft_App_WrCoCmd_Buffer(phost,BEGIN(LINES));
	for(i=-1;i<4;i++)
	{
		xoffset = i*136;
		yoffset = 0;
		x1offset = 136*(2 + i);
		y1offset = 272;
		if(x1offset > 480)
		{
			y1offset = 272 - (x1offset - 480);
			x1offset = 480;
		}
		if(xoffset < 0)
		{
			yoffset = -xoffset;
			xoffset = 0;
		}
		Ft_App_WrCoCmd_Buffer(phost,VERTEX2II(xoffset,yoffset,0,0));
		Ft_App_WrCoCmd_Buffer(phost,VERTEX2II(x1offset,y1offset,0,0));
	}
	for(i=1;i<6;i++)
	{
		xoffset = i*136;
		yoffset = 0;
		x1offset = 136*(i - 2);
		y1offset = 272;
		if(x1offset < 0)
		{
			y1offset = 272 + x1offset;
			x1offset = 0;
		}
		if(xoffset > 480)
		{
			yoffset = (xoffset - 480);
			xoffset = 480;
		}
		Ft_App_WrCoCmd_Buffer(phost,VERTEX2II(xoffset,yoffset,0,0));
		Ft_App_WrCoCmd_Buffer(phost,VERTEX2II(x1offset,y1offset,0,0));
	}
	if(!fg)
	{
		if(t<rate) t+=vel; else
		fg = 1;   		
	}else
	{
		if(t>0) t-=vel;
		else
		fg=0;
	}
	Ft_App_WrCoCmd_Buffer(phost,RESTORE_CONTEXT());
	return;
}
#endif

#ifdef BACKGROUND_ANIMATION_3
ft_uint16_t xoffset_array[30],dx_pts[20];
ft_uint8_t yoffset_array[30],bitmap_handle[30],dy_pts[20];
ft_uint8_t rate_cts[30],iteration_cts[30];

void Backgroundanimation_3()
{
	ft_int32_t i=0,rate = 1000;
	ft_uint8_t alpha  = 0,VEL = 0;
	static ft_uint16_t t1 = 0;
	ft_int16_t xoff = 0 ,yoff = 0;
	if(istouch())	
	{ 
		t1 = 0; VEL  = 2; 
	} 
	else {
		VEL = linear(2,1,t1,100); 
		if(t1<100) t1++; 
	}
	//clear the background color
	Ft_App_WrCoCmd_Buffer(phost,SAVE_CONTEXT());
	Ft_App_WrCoCmd_Buffer(phost,CLEAR_COLOR_RGB(0,0,0));
	Ft_App_WrCoCmd_Buffer(phost,CLEAR_COLOR_A(0));
	Ft_App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));
	Ft_App_WrCoCmd_Buffer(phost,COLOR_MASK(1,1,1,0));
	//draw the cmd gradient with scissors
	Ft_App_WrCoCmd_Buffer(phost,SCISSOR_SIZE(480,136));
	Ft_App_WrCoCmd_Buffer(phost,SCISSOR_XY(0,0));
	Ft_Gpu_CoCmd_Gradient(phost,0,0,0x708fa1,0,136,0xc4cdd2);
	Ft_App_WrCoCmd_Buffer(phost,SCISSOR_XY(0,136));
	Ft_Gpu_CoCmd_Gradient(phost,0,136,0xc4cdd2,0,272,0x4f7588);
	
	Ft_App_WrCoCmd_Buffer(phost,SCISSOR_XY(0,0));
	Ft_App_WrCoCmd_Buffer(phost,SCISSOR_SIZE(512,512));//reprogram with  default values

	//draw 20 points with various radious with additive blending
	Ft_App_WrCoCmd_Buffer(phost,BEGIN(FTPOINTS));	
	Ft_App_WrCoCmd_Buffer(phost,COLOR_MASK(1,1,1,0));
	Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
	for(i=0;i<20;i++)
	{
		//if(0 == i%4)		
		alpha  = linear(80,0,iteration_cts[i],rate_cts[i]);
		if(alpha<75)
		{
			Ft_App_WrCoCmd_Buffer(phost,POINT_SIZE(16*(30 + (3*i/2))));			
			Ft_App_WrCoCmd_Buffer(phost,COLOR_A(alpha));
			xoff = linear(xoffset_array[i],dx_pts[i],iteration_cts[i],rate_cts[i]);
			yoff = linear(yoffset_array[i],dy_pts[i],iteration_cts[i],rate_cts[i]);
			Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(xoff*16,yoff*16));
		}
	}
//	
	Ft_App_WrCoCmd_Buffer(phost,RESTORE_CONTEXT());
	for(i = 0; i<20 ; i++)
	{
		if(iteration_cts[i]==0)
		{
			xoffset_array[i] = ft_random(FT_DispWidth);
			yoffset_array[i] = 100+ft_random(FT_DispHeight/4);
			dx_pts[i] = ft_random(FT_DispWidth);
			dy_pts[i] = ft_random(255);				
			rate_cts[i] = 100+ft_random(155);
		}
		if(iteration_cts[i]<rate_cts[i])iteration_cts[i]+=VEL; else{ iteration_cts[i] = 0;}
	}

	return;
}
#endif

#ifdef BACKGROUND_ANIMATION_4

ft_uint16_t xoffset_array[30],dx_pts[20];
ft_uint8_t yoffset_array[30],bitmap_handle[30],dy_pts[20];
ft_uint8_t rate_cts[30],iteration_cts[30];

void Backgroundanimation_4()
{
	static char init = 0;
	ft_int32_t i,linesize;	
	ft_int16_t xoff = 0 ,yoff = 0;
	ft_uint8_t tval = 0,numBlobs=20,VEL = 0,alpha = 0;
	if(!init)
	{	
		/*Load background raw data*/
		byte file = imfile.openfile("hline.raw");
		ftsize = imfile.size; 
		Load_file2gram(255*1024L,1,imfile);
		init = 1;
	}

	if(istouch())	
		VEL = 2; else VEL = 1;

	for(i = 0; i<numBlobs ; i++)
	{
		if(iteration_cts[i]==0)
		{
			xoffset_array[i] = ft_random(FT_DispWidth);
			yoffset_array[i] = ft_random(FT_DispHeight);
			dx_pts[i] = ft_random(FT_DispWidth);
			dy_pts[i] = ft_random(FT_DispHeight);	
			rate_cts[i] = 500+ft_random(500);
		}
		if(iteration_cts[i]<rate_cts[i])iteration_cts[i]+=VEL; else{ iteration_cts[i] = 0;}
	}

	//draw the bitmap at location 0 with RGB565
	Ft_App_WrCoCmd_Buffer(phost,SAVE_CONTEXT());
	Ft_App_WrCoCmd_Buffer(phost,BITMAP_HANDLE(1));//bitmap handle 2 is used for background balls
	Ft_App_WrCoCmd_Buffer(phost,BITMAP_SOURCE(255*1024L));
	Ft_App_WrCoCmd_Buffer(phost,BITMAP_LAYOUT(L8, 512, 1));
	Ft_App_WrCoCmd_Buffer(phost,BITMAP_SIZE(NEAREST, BORDER, REPEAT, 512, 512));

	//clear the background color
	Ft_App_WrCoCmd_Buffer(phost,CLEAR_COLOR_RGB(255,255,255));
	Ft_App_WrCoCmd_Buffer(phost,CLEAR_COLOR_A(0));
	Ft_App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));
	Ft_App_WrCoCmd_Buffer(phost,COLOR_MASK(1,1,1,1));


	Ft_Gpu_CoCmd_LoadIdentity(phost);
	Ft_Gpu_CoCmd_Rotate(phost, 60*65536/360);//rotate by 30 degrees clock wise

	Ft_Gpu_CoCmd_SetMatrix(phost );

	Ft_App_WrCoCmd_Buffer(phost,COLOR_MASK(0,0,0,1));
	Ft_App_WrCoCmd_Buffer(phost,BLEND_FUNC(ONE,ZERO));
	Ft_App_WrCoCmd_Buffer(phost,BEGIN(BITMAPS));
	Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0xff,0xb4,0x00));
	Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
	Ft_App_WrCoCmd_Buffer(phost,VERTEX2II(0,0,1,0));	

	Ft_App_WrCoCmd_Buffer(phost,COLOR_MASK(1,1,1,1));
	Ft_App_WrCoCmd_Buffer(phost,BLEND_FUNC(DST_ALPHA,ONE_MINUS_DST_ALPHA));
	Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0xff,0xb4,0x00));
	Ft_App_WrCoCmd_Buffer(phost,BEGIN(RECTS));
	Ft_App_WrCoCmd_Buffer(phost,VERTEX2II(0,0,0,0));
	Ft_App_WrCoCmd_Buffer(phost,VERTEX2II(FT_DispWidth,FT_DispHeight,0,0));

	Ft_App_WrCoCmd_Buffer(phost,BEGIN(RECTS));
	Ft_App_WrCoCmd_Buffer(phost,BLEND_FUNC(SRC_ALPHA,ONE_MINUS_DST_ALPHA));
	Ft_App_WrCoCmd_Buffer(phost,COLOR_MASK(1,1,1,0));
	Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0x96,0x6e,0x0d));
	Ft_App_WrCoCmd_Buffer(phost,LINE_WIDTH(16*1));
	for(i=0;i<numBlobs;i++)
	{
		ft_int32_t xoffset,yoffset;
		if(0 == i%4)
		{
			linesize = 16*(25 + (3*i/4));
		}
		alpha  = linear(80,0,iteration_cts[i],rate_cts[i]);
		if(alpha<75)
		{
			xoff = linear(xoffset_array[i],dx_pts[i],iteration_cts[i],rate_cts[i]);
			yoff = linear(yoffset_array[i],dy_pts[i],iteration_cts[i],rate_cts[i]);
			Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(xoff*16,yoff*16));
			Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(xoff*16+linesize,yoff*16+linesize));
		}
	}
	Ft_App_WrCoCmd_Buffer(phost,RESTORE_CONTEXT());
}
#endif

#ifdef BACKGROUND_ANIMATION_5
static struct 
{
	ft_uint8_t init;
	ft_uint16_t xoffset_array[12];
	ft_uint16_t yoffset_array[12];
	ft_int16_t yoffset_array_source[12];
	ft_uint16_t iteration_cts[12];
	ft_uint8_t disable_cts[20][12];
	ft_uint8_t radius_a[12];
	ft_uint8_t radius_b[12];
	ft_uint16_t angle[12];
	ft_uint8_t number_of_firebubbles;
}firebubbles;


void draw_bubbles(ft_uint8_t inc)
{
	ft_int16_t i,j,yoff,xoff;
	for(j=0;j<3;j++)
	{
		Ft_App_WrCoCmd_Buffer(phost,CELL(j));
		if(j==2) 
			Ft_App_WrCoCmd_Buffer(phost,COLOR_A(150));
		else
			Ft_App_WrCoCmd_Buffer(phost,COLOR_A(255));
		for(i=0;i<firebubbles.number_of_firebubbles;i++)
		{
			if(firebubbles.iteration_cts[j*5+i] < firebubbles.yoffset_array[j*5+i]+0)
			{
				yoff = acceleration(firebubbles.yoffset_array_source[j*5+i],firebubbles.yoffset_array[j*5+i],firebubbles.iteration_cts[j*5+i],firebubbles.yoffset_array[j*5+i]*1);
				Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(firebubbles.xoffset_array[j*5+i]*16,yoff*16));

				if(inc){
				if(firebubbles.iteration_cts[j*5+i]<firebubbles.yoffset_array[j*5+i]*1)  firebubbles.iteration_cts[j*5+i]+=1;  					
				}
			}
		}
	}
}

ft_void_t collaid_bubbles(ft_uint8_t inc)
{
	ft_int16_t i,j,k,yoff,xoff,temp;
	static ft_uint8_t rate = 50;
	Ft_App_WrCoCmd_Buffer(phost,CELL(3));
	for(j=0;j<3;j++)
	{
		for(i=0;i<firebubbles.number_of_firebubbles;i++)
		{		
			if(firebubbles.iteration_cts[j*5+i]>=firebubbles.yoffset_array[j*5+i])
			{
				for(k=0;k<12;k++)	
				{
					Ft_App_WrCoCmd_Buffer(phost,COLOR_A(200-firebubbles.disable_cts[j*5+i][k]*10));		
					temp = (ft_uint8_t)deceleration(0,firebubbles.radius_a[k],firebubbles.disable_cts[j*5+i][k],20);
					xoff = firebubbles.xoffset_array[j*5+i]+10 + (temp)*cos(firebubbles.angle[k]*0.01744);  //3.14/180=0.01744		
					temp = (ft_uint8_t)deceleration(0,firebubbles.radius_b[k],firebubbles.disable_cts[j*5+i][k],20);
					yoff = firebubbles.yoffset_array[j*5+i]+10 + (temp)*sin(firebubbles.angle[k]*0.01744);  //3.14/180=0.01744
					yoff = firebubbles.yoffset_array[j*5+i]+10 + (temp)*sin(firebubbles.angle[k]*0.01744);  //3.14/180=0.01744
					Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(xoff*16,yoff*16));
					if(inc)
					{
						temp =  j*5+i;
						if(firebubbles.disable_cts[temp][k]<20)
						firebubbles.disable_cts[temp][k]++;
						else
						{
							firebubbles.disable_cts[temp][k] = 0;
							firebubbles.iteration_cts[temp] = 0;							
							firebubbles.xoffset_array[temp] = ft_random(FT_DispWidth);
							firebubbles.yoffset_array_source[temp] = -50-ft_random(100);
							if(j==0)
							firebubbles.yoffset_array[temp] = ft_random(20)+(FT_DispHeight-50); 		
							else if(j==1)
							firebubbles.yoffset_array[temp] = ft_random(20)+(FT_DispHeight-75); 		
							else if(j==2)
							firebubbles.yoffset_array[temp] = ft_random(20)+(FT_DispHeight-95); 		
							
						}
					}
				}
			}
		}
	}
}

ft_void_t Backgroundanimation_5()
{
	ft_int16_t i,j,yoff,xoff;
	firebubbles.number_of_firebubbles = 3;
	
	if(!firebubbles.init)
	{
		firebubbles.init = 1;		
		for(i=0;i<12;i++)
		{
			firebubbles.xoffset_array[i] = ft_random(FT_DispWidth);
			firebubbles.yoffset_array[i] = ft_random(50) + (FT_DispHeight-80);
			firebubbles.yoffset_array_source[i] =  -50 - ft_random(100);		
		}		
		for(i = 0;i<12;i++)
		{
				firebubbles.radius_a[i] = ft_random(70);
				firebubbles.radius_b[i] = ft_random(10);
				firebubbles.angle[i] =  ft_random(360);
		}
		
		byte file = imfile.openfile("fire.raw");
		ftsize = imfile.size; 		
		Load_file2gram(0,1,imfile);
		
		file = imfile.openfile("floor.raw");		
		ftsize = imfile.size; 		
		Load_file2gram(7*1024L,1,imfile);
		
		file = imfile.openfile("grad.raw");		
		ftsize = imfile.size; 		
		Load_file2gram(17*1024L,1,imfile);
		
		
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_HANDLE(5));	
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_SOURCE(0));	
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_LAYOUT(L8,40,40));
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_SIZE(NEAREST, BORDER, BORDER, 40, 40));
		
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_HANDLE(6));	
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_SOURCE(7*1024));	
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_LAYOUT(L8,240,40));
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_SIZE(NEAREST, BORDER, BORDER, 480, 80));
		
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_HANDLE(8));	
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_SOURCE(17*1024));	
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_LAYOUT(L8,60,30));
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_SIZE(NEAREST, BORDER, BORDER, 120, 60));
		for(i=0;i<80;i++)
		{
			Ft_Gpu_Hal_Wr8(phost,19*1024L+i,255-(i*2));	
		}
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_HANDLE(7));	
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_SOURCE(19*1024));	
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_LAYOUT(L8,1,80));
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_SIZE(NEAREST, REPEAT, BORDER, 480, 80));
	}	
	Ft_App_WrCoCmd_Buffer(phost,SAVE_CONTEXT());
	Ft_App_WrCoCmd_Buffer(phost,BEGIN(BITMAPS));
	Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(139,92,50));	
	
	Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_A(128));		
	Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_E(128));			
	Ft_App_WrCoCmd_Buffer(phost,VERTEX2II(0,FT_DispHeight-80,6,0));	
	Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_A(170));		
	Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_E(200));
	Ft_App_WrCoCmd_Buffer(phost,BLEND_FUNC(SRC_ALPHA,ONE));
	Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_A(256));		
	Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_E(256));
	Ft_App_WrCoCmd_Buffer(phost,BLEND_FUNC(SRC_ALPHA,ONE_MINUS_SRC_ALPHA));
	Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,0,0));	
	Ft_App_WrCoCmd_Buffer(phost,VERTEX2II(0,FT_DispHeight-80,7,0));
	Ft_App_WrCoCmd_Buffer(phost,BLEND_FUNC(SRC_ALPHA,ONE));
	Ft_App_WrCoCmd_Buffer(phost,BITMAP_HANDLE(5));	
	Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(200,10,10));	
	Ft_App_WrCoCmd_Buffer(phost,COLOR_A(200));	
	draw_bubbles(0);	
	Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,220,3));	
	Ft_App_WrCoCmd_Buffer(phost,COLOR_A(255));	
	draw_bubbles(1);
	Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(200,10,10));	
	collaid_bubbles(0);	
	Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,220,3));	
	collaid_bubbles(1);		
	Ft_App_WrCoCmd_Buffer(phost,RESTORE_CONTEXT());
}
#endif

#ifdef BACKGROUND_ANIMATION_6

#define ANGLE 18 
ft_int16_t xoffset_array[20],dx_pts[20],dy_pts[20];
ft_uint8_t yoffset_array[20],point_size[20];
ft_uint8_t rate_cts[20],iteration_cts[20];

void Backgroundanimation_6()
{
	static float move = 0;
	static ft_uint8_t init = 0;
	int z= 0,x=0,y=0;
	ft_int16_t Ox=0, Oy=0;
	ft_uint8_t VEL = 0;
	if(istouch()) VEL = 2; else VEL = 1;
	if(!init)
	{
		for(z = 0; z<20 ; z++)
		{
			point_size[z] = 20 +ft_random(20); 		
			dx_pts[z] = ft_random(FT_DispWidth+120);
			if(dx_pts[z]<FT_DispWidth)
			dy_pts[z] = -ft_random(FT_DispHeight);
			else
			dy_pts[z] = ft_random(FT_DispHeight);				
			rate_cts[z] = 100+ft_random(155);
		}
		init = 1;
	}
	
	move+=0.1; 
	if(move>=90)
	move = 0;
	Ft_App_WrCoCmd_Buffer(phost,SAVE_CONTEXT());
	Ft_Gpu_CoCmd_Gradient(phost,0, 0, 0x183c78, 340, 0,0x4560dd);
	Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
	Ft_App_WrCoCmd_Buffer(phost,COLOR_A(120));
	Ft_App_WrCoCmd_Buffer(phost,LINE_WIDTH(15*16));
	Ft_App_WrCoCmd_Buffer(phost,BEGIN(LINES));
	for(z=0;z<20;z++)
	{
		polar((ft_int32_t)(0),move+z*ANGLE,0,272);
		polar((ft_int32_t)(600),move+z*ANGLE,0,272);
	}	
	Ft_App_WrCoCmd_Buffer(phost,COLOR_A(60));
	Ft_App_WrCoCmd_Buffer(phost,BEGIN(FTPOINTS));
	Ft_App_WrCoCmd_Buffer(phost,POINT_SIZE(95*16));
	Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(0,272*16));	
	Ft_App_WrCoCmd_Buffer(phost,COLOR_A(200));
	for(z=0;z<20;z++)
	{
		Ft_App_WrCoCmd_Buffer(phost,POINT_SIZE(point_size[z]*16));
		Ox = linear(0,dx_pts[z],iteration_cts[z],rate_cts[z]);
		Oy = linear(272,dy_pts[z],iteration_cts[z],rate_cts[z]);
		Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(Ox*16,Oy*16));	
	}	
	Ft_App_WrCoCmd_Buffer(phost,RESTORE_CONTEXT());
	for(z = 0; z<20 ; z++)
	{		
		if(iteration_cts[z]<rate_cts[z])iteration_cts[z]+=VEL; else{ iteration_cts[z] = 0;}
	}	
}
#endif
/* API for android method*/

void show_icon(ft_uint8_t iconno)
{
	Ft_Play_Sound(0x51,100,108);
	do
	{
		Ft_Gpu_CoCmd_Dlstart(phost);   
		Ft_App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));
		#ifdef BACKGROUND_ANIMATION_1		
			Backgroundanimation_1();
		#endif
		#ifdef BACKGROUND_ANIMATION_2		
			Backgroundanimation_2();
		#endif
		#ifdef BACKGROUND_ANIMATION_3		
			Backgroundanimation_3();
		#endif
		#ifdef BACKGROUND_ANIMATION_4		
			Backgroundanimation_4();
		#endif
		#ifdef BACKGROUND_ANIMATION_5		
			Backgroundanimation_5();
		#endif		
		#ifdef BACKGROUND_ANIMATION_6		
			Backgroundanimation_6();
		#endif	
		Ft_App_WrCoCmd_Buffer(phost,BEGIN(BITMAPS));
		Ft_App_WrCoCmd_Buffer(phost,SAVE_CONTEXT());
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_HANDLE(0));		  
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_SIZE(NEAREST,BORDER,BORDER,200,100)); 
		Ft_App_WrCoCmd_Buffer(phost,CELL(iconno-1));
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_A(128));
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_E(128));
		Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(((FT_DispWidth-200)/2)*16,((FT_DispHeight-100)/2)*16));	
		Ft_App_WrCoCmd_Buffer(phost,RESTORE_CONTEXT());
		#ifdef BACKGROUND_ANIMATION_4
		Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,0,0));
		#endif
		Ft_App_WrCoCmd_Buffer(phost,TAG('H'));
		Ft_App_WrCoCmd_Buffer(phost,VERTEX2II(5,5,14,0));	
		Ft_App_WrCoCmd_Buffer(phost,DISPLAY());
		Ft_Gpu_CoCmd_Swap(phost);
		Ft_App_Flush_Co_Buffer(phost);
		Ft_Gpu_Hal_WaitCmdfifo_empty(phost);		
	}while(Read_keys()!='H');
	Ft_Play_Sound(0x51,100,108);
	scroller.vel = 0;
}

void android_menu()
{
	ft_uint8_t image_height = 50,image_width = 100;
	ft_uint8_t dt = 30,dx,dy;
	ft_uint8_t col,row,per_frame,noof_frame,current_frame=0;
	ft_uint8_t i,key_in=0,key_in_counts=0,temp=0;

	ft_int16_t Ox,Oy,sx,drag=0,prev=0,drag_dt=30,dragth=0;

// for points

	ft_uint8_t  point_offset,point_dt =15;

 
	dx = (dt*2)+image_width;	
	dy = (10*2)+image_height;
	col = FT_DispWidth/dx;
	row = 2;
	per_frame = col*row;
	noof_frame = (MAX_MENUS-1)/per_frame;

	point_offset = (FT_DispWidth-(noof_frame+1)*(MENU_POINTSIZE+point_dt))/2;

	scroller_init((FT_DispWidth*noof_frame)*16);

	while(1)
	{
		/*Read touch screen x varaiation and tag in*/
		sx =  Ft_Gpu_Hal_Rd16(phost,REG_TOUCH_SCREEN_XY + 2);
		key_in = Read_keys();
		
		/*Check if any tag in*/    
		if(sx!=NOTOUCH)			
		keyin_cts++;     	
		
		/*Move into the particular frame based on dragdt now 30pixels*/
		if(sx==NOTOUCH)
		{
			keyin_cts = 0; 		
			if(drag>((current_frame*FT_DispWidth)+drag_dt)) drag = min((current_frame+1)*FT_DispWidth,drag+15); 
			if(drag<((current_frame*FT_DispWidth)-drag_dt)) drag = max((current_frame-1)*FT_DispWidth,drag-15); 
			if(dragth==drag) current_frame = drag/FT_DispWidth;
			dragth = drag;
			scroller.vel = 0; 
			scroller.base = dragth*16;				// 16bit pre
		}
		/*if tag in but still pendown take a scroller basevalue*/
		else if(keyin_cts>KEYIN_COUNTS)
		{
			key_in = 0;
			drag = scroller.base>>4;
		}
		if(key_in==0)  scroller_run();

		/*Display list start*/   
		Ft_Gpu_CoCmd_Dlstart(phost);   
		Ft_App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));
		#ifdef BACKGROUND_ANIMATION_1		
		Backgroundanimation_1();
		#endif
		#ifdef BACKGROUND_ANIMATION_2		
				Backgroundanimation_2();
		#endif
		#ifdef BACKGROUND_ANIMATION_3		
				Backgroundanimation_3();
		#endif
		#ifdef BACKGROUND_ANIMATION_4		
				Backgroundanimation_4();
		#endif
		#ifdef BACKGROUND_ANIMATION_5		
				Backgroundanimation_5();
		#endif		
		#ifdef BACKGROUND_ANIMATION_6		
				Backgroundanimation_6();
		#endif	
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_HANDLE(0));
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_SIZE(NEAREST,BORDER,BORDER,100,50));
		Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(1));
		Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
		Ft_App_WrCoCmd_Buffer(phost,LINE_WIDTH(25));				// for rect	
		Ft_App_WrCoCmd_Buffer(phost,BEGIN(RECTS));	
		Ft_App_WrCoCmd_Buffer(phost,COLOR_A(150));
		Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(100,106,156));
		Oy = 10;
		for(i=0;i<=noof_frame;i++)
		{
			Ox = 10;  
			Ox+=(i*FT_DispWidth);
			Ox-=drag;
			if(i==0) Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(156,100,128));
			if(i==1) Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(100,106,156));
			if(i==2) Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(156,152,100));   
			Ft_App_WrCoCmd_Buffer(phost,VERTEX2F((Ox)*16,(Oy)*16));	
			Ft_App_WrCoCmd_Buffer(phost,VERTEX2F((Ox+FT_DispWidth-20)*16,(ft_int16_t)(FT_DispHeight*0.75)*16));					// i pixels wide than image width +1 
		}
		Ft_App_WrCoCmd_Buffer(phost,COLOR_A(255));
		Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255)); 
		for(i=0;i<MAX_MENUS;i++)
		{
			Ox = dt+dx*(i%col);                                          // Calculate the xoffsets
			Ox +=((i/per_frame)*FT_DispWidth);       
			Ox -= drag;        
			Oy = dt+(dy*((i/col)%row));
			if(Ox > (FT_DispWidth+dt)) 0;	
			else
			{
				Ft_App_WrCoCmd_Buffer(phost,VERTEX2F((Ox-1)*16,(Oy-1)*16));	
				Ft_App_WrCoCmd_Buffer(phost,VERTEX2F((image_width+Ox+1)*16,(image_height+Oy+1)*16));
			}					// i pixels wide than image width +1
		} 
		Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(1)); 
		Ft_App_WrCoCmd_Buffer(phost,BEGIN(BITMAPS));		
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_HANDLE(0));
		for(i=0;i<MAX_MENUS;i++)
		{
			Ox = dt+dx*(i%col);                                          // Calculate the xoffsets
			Ox +=((i/per_frame)*FT_DispWidth);       
			Ox -= drag;        
			Oy = dt+(dy*((i/col)%row));	
			if(Ox > (FT_DispWidth+dt) || Ox < -dx) 0;	
			else
			{
				Ft_App_WrCoCmd_Buffer(phost,CELL(i));
				Ft_App_WrCoCmd_Buffer(phost,TAG(i+1));	
				Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(Ox*16,Oy*16));	
			}
		}
		Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(0));

		// frame_no_points

		Ft_App_WrCoCmd_Buffer(phost,POINT_SIZE(MENU_POINTSIZE*16));
		Ft_App_WrCoCmd_Buffer(phost,BEGIN(FTPOINTS));				
		Ft_App_WrCoCmd_Buffer(phost,COLOR_A(50));	
		Oy = FT_DispHeight - 20;
		for(i=0;i<=noof_frame;i++)
		{
		  Ox = point_offset+(i*(MENU_POINTSIZE+point_dt));
		  Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(Ox*16,Oy*16));		
		}

		Ox = point_offset+(current_frame*(MENU_POINTSIZE+point_dt));
		Ft_App_WrCoCmd_Buffer(phost,COLOR_A(255));
		Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(Ox*16,Oy*16));	

		Ft_App_WrCoCmd_Buffer(phost,DISPLAY());
		Ft_Gpu_CoCmd_Swap(phost);
		Ft_App_Flush_Co_Buffer(phost);
		Ft_Gpu_Hal_WaitCmdfifo_empty(phost);	
		if(key_in>0 && key_in<=12 && !istouch())
		show_icon(key_in);
	}
}

// Api to Single Row//
void menu_loopback()
{
	ft_uint8_t keyin_cts = 0,temp_keyin;
	ft_uint8_t image_height = 50,image_width = 100; 
	ft_uint8_t dt = 30,dx,dy;
	ft_uint8_t per_frame,no_frames,key_in,current_frame; 
	ft_int16_t sx,drag,Oy,Ox,dragth,i;
	dx = (dt*2)+image_width;	
	dy = (10*2)+image_height;
	per_frame = FT_DispWidth/dx;
	no_frames = (MAX_MENUS-1)/per_frame;

	scroller_init((FT_DispWidth*no_frames)*16);
	while(1)
	{
/*Read touch screen x varaiation and tag in*/
              sx =  Ft_Gpu_Hal_Rd16(phost,REG_TOUCH_SCREEN_XY + 2);
              key_in =  Read_keys();

              /*Check if any tag in*/    
              if(sx!=NOTOUCH)      
              keyin_cts++;               
              /*Move into the particular frame based on dragdt now 30pixels*/
              if(sx==NOTOUCH)
              keyin_cts = 0;                           
              /*if tag in but still pendown take a scroller basevalue*/
              else if(keyin_cts>KEYIN_COUNTS)
              key_in = 0;
              
              if(key_in==0)scroller_run();
              drag = scroller.base>>4;   


		Ft_CmdBuffer_Index =0;
		Ft_Gpu_CoCmd_Dlstart(phost);   
		Ft_App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));
		#ifdef BACKGROUND_ANIMATION_1		
				Backgroundanimation_1();
		#endif
		#ifdef BACKGROUND_ANIMATION_2		
				Backgroundanimation_2();
		#endif
		#ifdef BACKGROUND_ANIMATION_3		
				Backgroundanimation_3();
		#endif
		#ifdef BACKGROUND_ANIMATION_4		
				Backgroundanimation_4();
		#endif
		#ifdef BACKGROUND_ANIMATION_5		
			Backgroundanimation_5();
		#endif		
		#ifdef BACKGROUND_ANIMATION_6		
			Backgroundanimation_6();
		#endif			
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_HANDLE(0));	 
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_SIZE(NEAREST, BORDER, BORDER, 100, 50));	
		Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(1));
		Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
		Ft_App_WrCoCmd_Buffer(phost,LINE_WIDTH(1*16)); // for rect	

		Oy = (FT_DispHeight-image_width)/2;					//dt+(dy*((i/col)%row));
		current_frame = drag/dx;                       // noof items moved in +/- directions
		dragth = drag%dx;

		Ft_App_WrCoCmd_Buffer(phost,BEGIN(RECTS));
		for(i=-1;i<(per_frame+1);i++)
		{
			Ox = dt+dx*i;
			Ox-=dragth;
			if(Ox > (FT_DispWidth+dt) || Ox < -dx) 0;						
			else
			{      
				Ft_App_WrCoCmd_Buffer(phost,VERTEX2F((Ox-1)*16,(Oy-1)*16));	
				Ft_App_WrCoCmd_Buffer(phost,VERTEX2F((image_width+Ox+1)*16,(image_height+Oy+1)*16));					// i pixels wide than image width +1  
			}
		}

		Ft_App_WrCoCmd_Buffer(phost,BEGIN(BITMAPS));											// draw the bitmap
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_HANDLE(0));
		for(i=-1;i<(per_frame+1);i++)
		{
			Ox = dt+dx*i;
			Ox-=dragth;
			if(Ox > (FT_DispWidth+dt) || Ox < -dx) 0;						
			else
			{ 
				Ft_App_WrCoCmd_Buffer(phost,CELL((MAX_MENUS+i+current_frame)%12));
				Ft_App_WrCoCmd_Buffer(phost,TAG((1+i+current_frame)%(MAX_MENUS+1)));	
				Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(Ox*16,Oy*16));	
			}
		}
		
		Ft_App_WrCoCmd_Buffer(phost,DISPLAY());
		Ft_Gpu_CoCmd_Swap(phost);
		Ft_App_Flush_Co_Buffer(phost);
		Ft_Gpu_Hal_WaitCmdfifo_empty(phost);	
		if(key_in>0 && key_in<=12 && !istouch())
		show_icon(key_in);
	}  
}  

// Api to tiles type menu*/
void menu_win8()
{
	ft_uint8_t current_frame = 0,total_frames = 0,key_in=0;	
	ft_int16_t frame_xoffset = 0,frame_xoffset_th= 0; 
	ft_uint8_t menus_per_frame = 0;
	ft_uint8_t col = 3,row = 2,option;
	ft_uint16_t image_height = 50,image_width = 100,rectangle_width,rectangle_height; 
	ft_int16_t Ox,Oy,i,sx;
#if defined(DISPLAY_RESOLUTION_WVGA)
	ft_uint8_t frame_xoffset_dt = 50;
#else
	ft_uint8_t frame_xoffset_dt = 30;
#endif


	ft_uint8_t color[12][3] = {  0xE0,0x01B,0xA2,
								 0x1B,0xE0,0xA8,
								 0x9E,0x9E,0x73,
								 0xE0,0x8E,0x1B,
								 0xB8,0x91,0xB3,
								 0x6E,0x96,0x8e,
								 0x1B,0x60,0xE0,
								 0xC7,0xE3,0x7B,
								 0x8B,0x1B,0xE0,
								 0xE3,0x91,0xC1,
								 0xE0,0x8E,0x1B,
								 0xAC,0x12,0xE3,
							};

	char *menudetails[20]=	{  "Music",  "Gauges ",  "Gradient",  "Photo",  "Metaball",  "Notepad",  "Signature",  "Sketch","Swiss","Waves","Player","Clocks"};

	ft_uint8_t  point_offset,frame_point_dt =15;

 
	ft_uint16_t dx = (frame_xoffset_dt*2)+image_width;	
	ft_uint16_t dy = (10*2)+image_height;
	col = FT_DispWidth/dx;
	menus_per_frame = col*row;
	total_frames = (MAX_MENUS-1)/menus_per_frame;

	point_offset = (FT_DispWidth-(total_frames+1)*(MENU_POINTSIZE+frame_point_dt))/2;	
	/*Load menu Thumbnails*/
	//Load_Thumbnails();
	/*Intilaize the scroller*/
	scroller_init((FT_DispWidth*total_frames)*16);


	while(1)
	{ 
		/*Read touch screen x varaiation and tag in*/
		sx =  Ft_Gpu_Hal_Rd16(phost,REG_TOUCH_SCREEN_XY + 2);
		key_in =  Read_keys();

		/*Check if any tag in*/    
		if(sx!=NOTOUCH)	
		keyin_cts++;     	
			   
		
		/*Move into the particular frame based on dragdt now 30pixels*/
		if(sx==NOTOUCH)
		{
			keyin_cts = 0;
			frame_xoffset = scroller.base>>4; 	
		}
		/*if tag in but still pendown take a scroller basevalue*/
		else if(keyin_cts>KEYIN_COUNTS)
		{
			key_in = 0;
			frame_xoffset = scroller.base>>4;
		}
		if(key_in==0)scroller_run();
		
	    Ft_Gpu_CoCmd_Dlstart(phost);   
	    Ft_App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_HANDLE(0));		  
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_SIZE(NEAREST,BORDER,BORDER,100,50)); 
	    Ft_App_WrCoCmd_Buffer(phost,SAVE_CONTEXT());
		#ifdef BACKGROUND_ANIMATION_1
				Backgroundanimation_1();
		#endif

		#ifdef BACKGROUND_ANIMATION_2
				Backgroundanimation_2();
		#endif
		#ifdef BACKGROUND_ANIMATION_3
				Backgroundanimation_3();
		#endif
		#ifdef BACKGROUND_ANIMATION_4
				Backgroundanimation_4();
		#endif

		#ifdef BACKGROUND_ANIMATION_5
				Backgroundanimation_5();
		#endif
		#ifdef BACKGROUND_ANIMATION_6		
				Backgroundanimation_6();
		#endif
	    Ft_App_WrCoCmd_Buffer(phost,RESTORE_CONTEXT());

#if defined(DISPLAY_RESOLUTION_WVGA) && defined(FT_81X_ENABLE)
		Ft_App_WrCoCmd_Buffer(phost, VERTEX_FORMAT(0));
#endif
	   	   
#if defined(DISPLAY_RESOLUTION_WQVGA) || defined(DISPLAY_RESOLUTION_WVGA)
		for(option=0;option<3;option++)
		{
		   switch(option)
		   {
			  case 0:
				 Ft_App_WrCoCmd_Buffer(phost,LINE_WIDTH(1*16));
				 Ft_App_WrCoCmd_Buffer(phost,BEGIN(RECTS));
			  break;
			  case 1:
				  Ft_App_WrCoCmd_Buffer(phost,BEGIN(BITMAPS));
				  Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
			  break;
			  case 2:	  
				  Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
			  break;
		   }
#if defined(DISPLAY_RESOLUTION_WVGA)
		   //rectangle_width = 366;
		   rectangle_width = 360;
		   rectangle_height = 200;
#else
		   rectangle_width = 220;
		   rectangle_height = 100;
#endif
		   for(i=0;i<4;i+=1)
		   {
				if(i<2)
				{
					Ox = 10+FT_DispWidth*i;
					//Ox = 0 +FT_DispWidth*i;
					Oy = 10;
				}else
				{	
#if defined(DISPLAY_RESOLUTION_WVGA)
					Ox = 400+FT_DispWidth*(i%2);
					Oy = 220;
#else
					Ox = 250+FT_DispWidth*(i%2);
					Oy = 120;
#endif
				}
				Ox -= frame_xoffset;		
				//if(Ox > (FT_DispWidth+frame_xoffset_dt) || Ox < -512) 0;	
				if(Ox > (FT_DispWidth+frame_xoffset_dt) || Ox < (-FT_DispWidth)) 0;
				else
				{	
					Ft_App_WrCoCmd_Buffer(phost,TAG(i+1));	
					switch(option)
					{
						case 0:
							Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(color[i][0],color[i][1],color[i][2]));	
							#if defined(DISPLAY_RESOLUTION_WVGA)
								Ft_App_WrCoCmd_Buffer(phost,VERTEX2F((Ox),(Oy)));	
								Ft_App_WrCoCmd_Buffer(phost,VERTEX2F((rectangle_width+Ox),(rectangle_height+Oy)));					// i pixels wide than image width +1
							#else
								Ft_App_WrCoCmd_Buffer(phost,VERTEX2F((Ox)*16,(Oy)*16));	
								Ft_App_WrCoCmd_Buffer(phost,VERTEX2F((rectangle_width+Ox)*16,(rectangle_height+Oy)*16));	
							#endif
						break;
						case 1:
							Ft_App_WrCoCmd_Buffer(phost,CELL(i));
							#if defined(DISPLAY_RESOLUTION_WVGA)
								Ft_App_WrCoCmd_Buffer(phost,VERTEX2F((55+Ox),(25+Oy)));
							#else
								Ft_App_WrCoCmd_Buffer(phost,VERTEX2F((55+Ox)*16,(25+Oy)*16));
							#endif
						break;

						case 2:
							Ft_Gpu_CoCmd_Text(phost,Ox+10,Oy+80,26,0,menudetails[i]);
						break;
					}
				}
		   }
#if defined(DISPLAY_RESOLUTION_WVGA)
		   rectangle_width = 170;
		   rectangle_height = 200;	
#else
		   rectangle_width = 100;
		   rectangle_height = 100;	
#endif
		   if(option==1)  Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_A(512));
		  
		   for(i=0;i<8;i+=1)
		   {
				if(i<4)
				{
#if defined(DISPLAY_RESOLUTION_WVGA)  
					//Ox = 400+FT_DispWidth*(i/2)+(image_width*(i%2))+(20*(i%2));	// 20 is space between two icon
					Ox = 400+FT_DispWidth*(i/2)+(rectangle_width*(i%2))+(20*(i%2));
					Oy = 10;
#else
					Ox = 250+FT_DispWidth*(i/2)+(image_width*(i%2))+(20*(i%2));	// 20 is space between two icon
					//Ox = 150+FT_DispWidth*(i/2)+(image_width*(i%2))+(20*(i%2));
					Oy = 10;
#endif
				}
				else
				{	
#if defined(DISPLAY_RESOLUTION_WVGA)

					//Ox = 10 + (FT_DispWidth*(i/6)) + (((i-4)%2)*image_width) + (((i-4)%2)*20);
					Ox = 10 + (FT_DispWidth*(i/6)) + (((i-4)%2)*rectangle_width) + (((i-4)%2)*20);
					Oy = 220;
#else
					Ox = 10+FT_DispWidth*(i/6)+ (((i-4)%2)*image_width)+(((i-4)%2)*20);
					Oy = 120;
#endif
				}
				Ox -= frame_xoffset;		
				//if(Ox > (FT_DispWidth+frame_xoffset_dt) || Ox < -512) 0;	
				if(Ox > (FT_DispWidth+frame_xoffset_dt) || Ox < -FT_DispWidth) 0;	
				else
				{
					Ft_App_WrCoCmd_Buffer(phost,TAG(i+5));	
					switch(option)
					{	
						case 0:
								Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(color[i+5][0],color[i+5][1],color[i+5][2]));	
								#if defined(DISPLAY_RESOLUTION_WVGA)
									Ft_App_WrCoCmd_Buffer(phost,VERTEX2F((Ox),(Oy)));	
									Ft_App_WrCoCmd_Buffer(phost,VERTEX2F((rectangle_width+Ox),(rectangle_height+Oy)));					// i pixels wide than image width +1
								#else
									Ft_App_WrCoCmd_Buffer(phost,VERTEX2F((Ox)*16,(Oy)*16));	
									Ft_App_WrCoCmd_Buffer(phost,VERTEX2F((rectangle_width+Ox)*16,(rectangle_height+Oy)*16));	
								#endif
						break;

						case 1:							
								Ft_App_WrCoCmd_Buffer(phost,CELL(i+4));		
								#if defined(DISPLAY_RESOLUTION_WVGA)
									Ft_App_WrCoCmd_Buffer(phost,VERTEX2F((25+Ox),(25+Oy)));	
								#else
									Ft_App_WrCoCmd_Buffer(phost,VERTEX2F((25+Ox)*16,(25+Oy)*16));
								#endif
						break;

						case 2:
								Ft_Gpu_CoCmd_Text(phost,Ox+10,Oy+80,26,0,menudetails[i+4]);
						break;
					}								// i pixels wide than image width +1
				}
		   }
		   if(option==1)  Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_A(256));
		}			
  
#elif defined(DISPLAY_RESOLUTION_HVGA_PORTRAIT) || defined(DISPLAY_RESOLUTION_QVGA)
		
  rectangle_width = 170;
  rectangle_height = 100;	
  Ft_App_WrCoCmd_Buffer(phost,BEGIN(RECTS));
  for(i=0;i<12;i+=2)
  {
    Ox = 10+FT_DispWidth*(i/4);
    Ox -= frame_xoffset;
    Oy = 10+(110*((i/col)%row));
    Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(color[i][0],color[i][1],color[i][2]));
	Ft_App_WrCoCmd_Buffer(phost,TAG(i+1));	
    Ft_App_WrCoCmd_Buffer(phost,VERTEX2F((Ox)*16,(Oy)*16)	);	
    Ft_App_WrCoCmd_Buffer(phost,VERTEX2F((rectangle_width+Ox)*16,(rectangle_height+Oy)*16));					// i pixels wide than image width +1
  }

  rectangle_width = 110;
  rectangle_height = 100;	

  for(i=1;i<12;i+=2)
  {
    Ox = 200+FT_DispWidth*(i/4);
    Ox -= frame_xoffset;
    Oy = 10+(110*((i/col)%row));
    Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(color[i][0],color[i][1],color[i][2]));
	Ft_App_WrCoCmd_Buffer(phost,TAG(i+1));	
    Ft_App_WrCoCmd_Buffer(phost,VERTEX2F((Ox)*16,(Oy)*16));	
    Ft_App_WrCoCmd_Buffer(phost,VERTEX2F((rectangle_width+Ox)*16,(rectangle_height+Oy)*16));					// i pixels wide than image width +1
  }
   Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(0));

  Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
  for(i=0;i<12;i+=2)
  {
    Ox = 10+FT_DispWidth*(i/4);
    Ox -= frame_xoffset;
    Oy = 10+(110*((i/col)%row));
    Ft_Gpu_CoCmd_Text(phost,Ox+10,Oy+80,26,0,menudetails[i]);
  }
  for(i=1;i<12;i+=2)
  {
    Ox = 200+FT_DispWidth*(i/4);
    Ox -= frame_xoffset;
    Oy = 10+(110*((i/col)%row));
    Ft_Gpu_CoCmd_Text(phost,Ox+10,Oy+80,26,0,menudetails[i]);
  }
		
  rectangle_height = 100;
  rectangle_height = 50;
  Ft_App_WrCoCmd_Buffer(phost,BEGIN(BITMAPS));											// draw the bitmap
  Ft_App_WrCoCmd_Buffer(phost,BITMAP_HANDLE(0));
  for(i=0;i<12;i+=2)
  {
    Ox = 75+FT_DispWidth*(i/4);
    Ox -= frame_xoffset;
    Oy = 20+(110*((i/col)%row));
    Ft_App_WrCoCmd_Buffer(phost,CELL(i));
    Ft_App_WrCoCmd_Buffer(phost,TAG(i+1));	
    Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(Ox*16,Oy*16));	
  }
  Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_A(512));
  for(i=1;i<12;i+=2)
  {
    Ox = 230+FT_DispWidth*(i/4);
    Ox -= frame_xoffset;
    Oy = 20+(110*((i/col)%row));
    Ft_App_WrCoCmd_Buffer(phost,CELL(i));    
    Ft_App_WrCoCmd_Buffer(phost,TAG(i+1));	
    Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(Ox*16,Oy*16));	
  }
  Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_A(256));
#endif
#if defined(DISPLAY_RESOLUTION_WVGA) && defined(FT_81X_ENABLE)
  Ft_App_WrCoCmd_Buffer(phost, VERTEX_FORMAT(4));
#endif
  Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(0));

  Ft_App_WrCoCmd_Buffer(phost,DISPLAY());
  Ft_Gpu_CoCmd_Swap(phost);
  Ft_App_Flush_Co_Buffer(phost);
  Ft_Gpu_Hal_WaitCmdfifo_empty(phost);	 

  if(key_in!=0 && key_in<=12 && !istouch())
  show_icon(key_in);
  }

}



#ifdef FT900_PLATFORM
ft_void_t FT900_Config()
{
	sys_enable(sys_device_uart0);
    gpio_function(48, pad_uart0_txd); /* UART0 TXD */
    gpio_function(49, pad_uart0_rxd); /* UART0 RXD */
    uart_open(UART0,                    /* Device */
              1,                        /* Prescaler = 1 */
              UART_DIVIDER_115200_BAUD,  /* Divider = 1302 */
              uart_data_bits_8,         /* No. Data Bits */
              uart_parity_none,         /* Parity */
              uart_stop_bits_1);        /* No. Stop Bits */

    /* Print out a welcome message... */
    uart_puts(UART0,

        "(C) Copyright 2014-2015, Future Technology Devices International Ltd. \r\n"
        "--------------------------------------------------------------------- \r\n"
        "Welcome to Mainmenu Example ... \r\n"
        "\r\n"
        "--------------------------------------------------------------------- \r\n"
        );

#ifdef ENABLE_ILI9488_HVGA_PORTRAIT
/* asign all the respective pins to gpio and set them to default values */
gpio_function(34, pad_gpio34);
gpio_dir(34, pad_dir_output);
gpio_write(34,1);

gpio_function(27, pad_gpio27);
gpio_dir(27, pad_dir_output);
gpio_write(27,1);

gpio_function(29, pad_gpio29);
gpio_dir(29, pad_dir_output);
gpio_write(29,1);

gpio_function(33, pad_gpio33);
gpio_dir(33, pad_dir_output);
gpio_write(33,1);


gpio_function(30, pad_gpio30);
gpio_dir(30, pad_dir_output);
gpio_write(30,1);

gpio_function(28, pad_gpio28);
gpio_dir(28, pad_dir_output);
gpio_write(28,1);


	gpio_function(43, pad_gpio43);
	gpio_dir(43, pad_dir_output);
gpio_write(43,1);
gpio_write(34,1);
gpio_write(28,1);
gpio_write(43,1);
gpio_write(33,1);
gpio_write(33,1);

#endif
/* useful for timer */
//ft_millis_init();
//interrupt_enable_globally();
//printf("ft900 config done \n");
}
#endif





#if defined(MSVC_PLATFORM) || defined(FT900_PLATFORM)
/* Main entry point */
ft_int32_t main(ft_int32_t argc,ft_char8_t *argv[])
#endif
#if defined(ARDUINO_PLATFORM)||defined(MSVC_FT800EMU)
ft_void_t setup()
#endif
{	
#ifdef FT900_PLATFORM
	FT900_Config();

	sys_enable(sys_device_sd_card);
	sdhost_init();

			#define GPIO_SD_CLK  (19)
			#define GPIO_SD_CMD  (20)
			#define GPIO_SD_DAT3 (21)
			#define GPIO_SD_DAT2 (22)
			#define GPIO_SD_DAT1 (23)
			#define GPIO_SD_DAT0 (24)
			#define GPIO_SD_CD   (25)
			#define GPIO_SD_WP   (26)
		    gpio_function(GPIO_SD_CLK, pad_sd_clk); gpio_pull(GPIO_SD_CLK, pad_pull_none);//pad_pull_none
		    gpio_function(GPIO_SD_CMD, pad_sd_cmd); gpio_pull(GPIO_SD_CMD, pad_pull_pullup);
		    gpio_function(GPIO_SD_DAT3, pad_sd_data3); gpio_pull(GPIO_SD_DAT3, pad_pull_pullup);
		    gpio_function(GPIO_SD_DAT2, pad_sd_data2); gpio_pull(GPIO_SD_DAT2, pad_pull_pullup);
		    gpio_function(GPIO_SD_DAT1, pad_sd_data1); gpio_pull(GPIO_SD_DAT1, pad_pull_pullup);
		    gpio_function(GPIO_SD_DAT0, pad_sd_data0); gpio_pull(GPIO_SD_DAT0, pad_pull_pullup);
		    gpio_function(GPIO_SD_CD, pad_sd_cd); gpio_pull(GPIO_SD_CD, pad_pull_pullup);
		    gpio_function(GPIO_SD_WP, pad_sd_wp); gpio_pull(GPIO_SD_WP, pad_pull_pullup);
#endif
	Ft_Gpu_HalInit_t halinit;

    halinit.TotalChannelNum = 1;


	Ft_Gpu_Hal_Init(&halinit);
	host.hal_config.channel_no = 0;
	host.hal_config.pdn_pin_no = FT800_PD_N;
	host.hal_config.spi_cs_pin_no = FT800_SEL_PIN;
#ifdef MSVC_PLATFORM_SPI
    host.hal_config.spi_clockrate_khz = 12000; //in KHz
#endif
#ifdef ARDUINO_PLATFORM_SPI
    host.hal_config.spi_clockrate_khz = 4000; //in KHz
#endif
        Ft_Gpu_Hal_Open(&host);
    phost = &host;

    Ft_BootupConfig();

#if ((defined FT900_PLATFORM) || defined(MSVC_PLATFORM))
	printf("\n reg_touch_rz =0x%x ", Ft_Gpu_Hal_Rd16(phost, REG_TOUCH_RZ));
	printf("\n reg_touch_rzthresh =0x%x ", Ft_Gpu_Hal_Rd32(phost, REG_TOUCH_RZTHRESH));
  printf("\n reg_touch_tag_xy=0x%x",Ft_Gpu_Hal_Rd32(phost, REG_TOUCH_TAG_XY));
	printf("\n reg_touch_tag=0x%x",Ft_Gpu_Hal_Rd32(phost, REG_TOUCH_TAG));
#endif
#ifdef FT900_PLATFORM
	  SDHOST_STATUS sd_status;
	  //
	if(sdhost_card_detect() == SDHOST_CARD_INSERTED)
	{ printf("-------------1---------------");
	printf("\n sd_status=%s",sdhost_card_detect());
	}

		if (sdhost_card_detect() != SDHOST_CARD_INSERTED)
		{
		printf("Please Insert SD Card\r\n");
		}
		else
		{ printf("\n SD Card inserted!");}


		if (f_mount(&FatFs, "", 0) != FR_OK)
		{
			printf("\n Mounted failed.");;
		}

			printf("\n Mounted succesfully.");

#endif

    /*It is optional to clear the screen here*/
    Ft_Gpu_Hal_WrMem(phost, RAM_DL,(ft_uint8_t *)FT_DLCODE_BOOTUP,sizeof(FT_DLCODE_BOOTUP));
    Ft_Gpu_Hal_Wr8(phost, REG_DLSWAP,DLSWAP_FRAME);

    Ft_Gpu_Hal_Sleep(1000);//Show the booting up screen.
#if defined(ARDUINO_PLATFORM)
  pinMode(FT_SDCARD_CS, OUTPUT);
  digitalWrite(FT_SDCARD_CS, HIGH);
  delay(100);
  sd_present =  SD.begin(FT_SDCARD_CS);
  
  SPI.setClockDivider(SPI_CLOCK_DIV2);
  SPI.setBitOrder(MSBFIRST);
  SPI.setDataMode(SPI_MODE0);
#endif
	home_setup();
	Info();
	Load_Thumbnails();
	#if defined(ANDROID_METHOD)
	android_menu();
	#elif defined(LOOPBACK_METHOD)
	menu_loopback();
	#elif defined(WIN8_METHOD)
	menu_win8();
	#endif

    /* Close all the opened handles */
    Ft_Gpu_Hal_Close(phost);
    Ft_Gpu_Hal_DeInit();
#if  defined(MSVC_PLATFORM) || defined(FT900_PLATFORM)
    return 0;
#endif
}


void loop()
{
}



/* Nothing beyond this */













